@@ -1,6 +1,6 @@
 # Natural Language Toolkit: Shift-Reduce Parser Application
 #
-# Copyright (C) 2001-2017 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
@@ -59,8 +59,18 @@ Keyboard Shortcuts::
       [h]\t Help
       [Ctrl-p]\t Print
       [q]\t Quit
+
 """
 
+from six.moves.tkinter_font import Font
+from six.moves.tkinter import IntVar, Listbox, Button, Frame, Label, Menu, Scrollbar, Tk
+
+from nltk.tree import Tree
+from nltk.parse import SteppingShiftReduceParser
+from nltk.util import in_idle
+from nltk.draw.util import CanvasFrame, EntryDialog, ShowText, TextWidget
+from nltk.draw import CFGEditor, TreeSegmentWidget, tree_to_treesegment
+
 """
 Possible future improvements:
   - button/window to change and/or select text.  Just pop up a window
@@ -73,16 +83,6 @@ Possible future improvements:
     responsible for that.
 """
 
-import nltk.compat
-import tkinter.font
-from tkinter import (IntVar, Listbox, Button, Frame, Label, Menu,
-                     Scrollbar, Tk)
-
-from nltk.tree import Tree
-from nltk.parse import SteppingShiftReduceParser
-from nltk.util import in_idle
-from nltk.draw.util import CanvasFrame, EntryDialog, ShowText, TextWidget
-from nltk.draw import CFGEditor, TreeSegmentWidget, tree_to_treesegment
 
 class ShiftReduceApp(object):
     """
@@ -94,6 +94,7 @@ class ShiftReduceApp(object):
     the parsing process, performing the operations that
     ``nltk.parse.ShiftReduceParser`` would use.
     """
+
     def __init__(self, grammar, sent, trace=0):
         self._sent = sent
         self._parser = SteppingShiftReduceParser(grammar, trace)
@@ -106,7 +107,7 @@ class ShiftReduceApp(object):
         # from performing new operations while it's animating.
         self._animating_lock = 0
         self._animate = IntVar(self._top)
-        self._animate.set(10) # = medium
+        self._animate.set(10)  # = medium
 
         # The user can hide the grammar.
         self._show_grammar = IntVar(self._top)
@@ -138,32 +139,34 @@ class ShiftReduceApp(object):
 
     def _init_fonts(self, root):
         # See: <http://www.astro.washington.edu/owen/ROTKFolklore.html>
-        self._sysfont = tkinter.font.Font(font=Button()["font"])
+        self._sysfont = Font(font=Button()["font"])
         root.option_add("*Font", self._sysfont)
 
         # TWhat's our font size (default=same as sysfont)
         self._size = IntVar(root)
         self._size.set(self._sysfont.cget('size'))
 
-        self._boldfont = tkinter.font.Font(family='helvetica', weight='bold',
-                                    size=self._size.get())
-        self._font = tkinter.font.Font(family='helvetica',
-                                    size=self._size.get())
+        self._boldfont = Font(family='helvetica', weight='bold', size=self._size.get())
+        self._font = Font(family='helvetica', size=self._size.get())
 
     def _init_grammar(self, parent):
         # Grammar view.
         self._prodframe = listframe = Frame(parent)
         self._prodframe.pack(fill='both', side='left', padx=2)
-        self._prodlist_label = Label(self._prodframe,
-                                     font=self._boldfont,
-                                     text='Available Reductions')
+        self._prodlist_label = Label(
+            self._prodframe, font=self._boldfont, text='Available Reductions'
+        )
         self._prodlist_label.pack()
-        self._prodlist = Listbox(self._prodframe, selectmode='single',
-                                 relief='groove', background='white',
-                                 foreground='#909090',
-                                 font=self._font,
-                                 selectforeground='#004040',
-                                 selectbackground='#c0f0c0')
+        self._prodlist = Listbox(
+            self._prodframe,
+            selectmode='single',
+            relief='groove',
+            background='white',
+            foreground='#909090',
+            font=self._font,
+            selectforeground='#004040',
+            selectbackground='#c0f0c0',
+        )
 
         self._prodlist.pack(side='right', fill='both', expand=1)
 
@@ -173,10 +176,9 @@ class ShiftReduceApp(object):
         self._prodlist.config(height=min(len(self._productions), 25))
 
         # Add a scrollbar if there are more than 25 productions.
-        if 1:#len(self._productions) > 25:
-            listscroll = Scrollbar(self._prodframe,
-                                   orient='vertical')
-            self._prodlist.config(yscrollcommand = listscroll.set)
+        if 1:  # len(self._productions) > 25:
+            listscroll = Scrollbar(self._prodframe, orient='vertical')
+            self._prodlist.config(yscrollcommand=listscroll.set)
             listscroll.config(command=self._prodlist.yview)
             listscroll.pack(side='left', fill='y')
 
@@ -218,98 +220,173 @@ class ShiftReduceApp(object):
         self._top.bind('<Control-t>', self.edit_sentence)
 
         # Animation speed control
-        self._top.bind('-', lambda e,a=self._animate:a.set(20))
-        self._top.bind('=', lambda e,a=self._animate:a.set(10))
-        self._top.bind('+', lambda e,a=self._animate:a.set(4))
+        self._top.bind('-', lambda e, a=self._animate: a.set(20))
+        self._top.bind('=', lambda e, a=self._animate: a.set(10))
+        self._top.bind('+', lambda e, a=self._animate: a.set(4))
 
     def _init_buttons(self, parent):
         # Set up the frames.
         self._buttonframe = buttonframe = Frame(parent)
         buttonframe.pack(fill='none', side='bottom')
-        Button(buttonframe, text='Step',
-               background='#90c0d0', foreground='black',
-               command=self.step,).pack(side='left')
-        Button(buttonframe, text='Shift', underline=0,
-               background='#90f090', foreground='black',
-               command=self.shift).pack(side='left')
-        Button(buttonframe, text='Reduce', underline=0,
-               background='#90f090', foreground='black',
-               command=self.reduce).pack(side='left')
-        Button(buttonframe, text='Undo', underline=0,
-               background='#f0a0a0', foreground='black',
-               command=self.undo).pack(side='left')
+        Button(
+            buttonframe,
+            text='Step',
+            background='#90c0d0',
+            foreground='black',
+            command=self.step,
+        ).pack(side='left')
+        Button(
+            buttonframe,
+            text='Shift',
+            underline=0,
+            background='#90f090',
+            foreground='black',
+            command=self.shift,
+        ).pack(side='left')
+        Button(
+            buttonframe,
+            text='Reduce',
+            underline=0,
+            background='#90f090',
+            foreground='black',
+            command=self.reduce,
+        ).pack(side='left')
+        Button(
+            buttonframe,
+            text='Undo',
+            underline=0,
+            background='#f0a0a0',
+            foreground='black',
+            command=self.undo,
+        ).pack(side='left')
 
     def _init_menubar(self, parent):
         menubar = Menu(parent)
 
         filemenu = Menu(menubar, tearoff=0)
-        filemenu.add_command(label='Reset Parser', underline=0,
-                             command=self.reset, accelerator='Del')
-        filemenu.add_command(label='Print to Postscript', underline=0,
-                             command=self.postscript, accelerator='Ctrl-p')
-        filemenu.add_command(label='Exit', underline=1,
-                             command=self.destroy, accelerator='Ctrl-x')
+        filemenu.add_command(
+            label='Reset Parser', underline=0, command=self.reset, accelerator='Del'
+        )
+        filemenu.add_command(
+            label='Print to Postscript',
+            underline=0,
+            command=self.postscript,
+            accelerator='Ctrl-p',
+        )
+        filemenu.add_command(
+            label='Exit', underline=1, command=self.destroy, accelerator='Ctrl-x'
+        )
         menubar.add_cascade(label='File', underline=0, menu=filemenu)
 
         editmenu = Menu(menubar, tearoff=0)
-        editmenu.add_command(label='Edit Grammar', underline=5,
-                             command=self.edit_grammar,
-                             accelerator='Ctrl-g')
-        editmenu.add_command(label='Edit Text', underline=5,
-                             command=self.edit_sentence,
-                             accelerator='Ctrl-t')
+        editmenu.add_command(
+            label='Edit Grammar',
+            underline=5,
+            command=self.edit_grammar,
+            accelerator='Ctrl-g',
+        )
+        editmenu.add_command(
+            label='Edit Text',
+            underline=5,
+            command=self.edit_sentence,
+            accelerator='Ctrl-t',
+        )
         menubar.add_cascade(label='Edit', underline=0, menu=editmenu)
 
         rulemenu = Menu(menubar, tearoff=0)
-        rulemenu.add_command(label='Step', underline=1,
-                             command=self.step, accelerator='Space')
+        rulemenu.add_command(
+            label='Step', underline=1, command=self.step, accelerator='Space'
+        )
         rulemenu.add_separator()
-        rulemenu.add_command(label='Shift', underline=0,
-                             command=self.shift, accelerator='Ctrl-s')
-        rulemenu.add_command(label='Reduce', underline=0,
-                             command=self.reduce, accelerator='Ctrl-r')
+        rulemenu.add_command(
+            label='Shift', underline=0, command=self.shift, accelerator='Ctrl-s'
+        )
+        rulemenu.add_command(
+            label='Reduce', underline=0, command=self.reduce, accelerator='Ctrl-r'
+        )
         rulemenu.add_separator()
-        rulemenu.add_command(label='Undo', underline=0,
-                             command=self.undo, accelerator='Ctrl-u')
+        rulemenu.add_command(
+            label='Undo', underline=0, command=self.undo, accelerator='Ctrl-u'
+        )
         menubar.add_cascade(label='Apply', underline=0, menu=rulemenu)
 
         viewmenu = Menu(menubar, tearoff=0)
-        viewmenu.add_checkbutton(label="Show Grammar", underline=0,
-                                 variable=self._show_grammar,
-                                 command=self._toggle_grammar)
+        viewmenu.add_checkbutton(
+            label="Show Grammar",
+            underline=0,
+            variable=self._show_grammar,
+            command=self._toggle_grammar,
+        )
         viewmenu.add_separator()
-        viewmenu.add_radiobutton(label='Tiny', variable=self._size,
-                                 underline=0, value=10, command=self.resize)
-        viewmenu.add_radiobutton(label='Small', variable=self._size,
-                                 underline=0, value=12, command=self.resize)
-        viewmenu.add_radiobutton(label='Medium', variable=self._size,
-                                 underline=0, value=14, command=self.resize)
-        viewmenu.add_radiobutton(label='Large', variable=self._size,
-                                 underline=0, value=18, command=self.resize)
-        viewmenu.add_radiobutton(label='Huge', variable=self._size,
-                                 underline=0, value=24, command=self.resize)
+        viewmenu.add_radiobutton(
+            label='Tiny',
+            variable=self._size,
+            underline=0,
+            value=10,
+            command=self.resize,
+        )
+        viewmenu.add_radiobutton(
+            label='Small',
+            variable=self._size,
+            underline=0,
+            value=12,
+            command=self.resize,
+        )
+        viewmenu.add_radiobutton(
+            label='Medium',
+            variable=self._size,
+            underline=0,
+            value=14,
+            command=self.resize,
+        )
+        viewmenu.add_radiobutton(
+            label='Large',
+            variable=self._size,
+            underline=0,
+            value=18,
+            command=self.resize,
+        )
+        viewmenu.add_radiobutton(
+            label='Huge',
+            variable=self._size,
+            underline=0,
+            value=24,
+            command=self.resize,
+        )
         menubar.add_cascade(label='View', underline=0, menu=viewmenu)
 
         animatemenu = Menu(menubar, tearoff=0)
-        animatemenu.add_radiobutton(label="No Animation", underline=0,
-                                    variable=self._animate, value=0)
-        animatemenu.add_radiobutton(label="Slow Animation", underline=0,
-                                    variable=self._animate, value=20,
-                                    accelerator='-')
-        animatemenu.add_radiobutton(label="Normal Animation", underline=0,
-                                    variable=self._animate, value=10,
-                                    accelerator='=')
-        animatemenu.add_radiobutton(label="Fast Animation", underline=0,
-                                    variable=self._animate, value=4,
-                                    accelerator='+')
+        animatemenu.add_radiobutton(
+            label="No Animation", underline=0, variable=self._animate, value=0
+        )
+        animatemenu.add_radiobutton(
+            label="Slow Animation",
+            underline=0,
+            variable=self._animate,
+            value=20,
+            accelerator='-',
+        )
+        animatemenu.add_radiobutton(
+            label="Normal Animation",
+            underline=0,
+            variable=self._animate,
+            value=10,
+            accelerator='=',
+        )
+        animatemenu.add_radiobutton(
+            label="Fast Animation",
+            underline=0,
+            variable=self._animate,
+            value=4,
+            accelerator='+',
+        )
         menubar.add_cascade(label="Animate", underline=1, menu=animatemenu)
 
-
         helpmenu = Menu(menubar, tearoff=0)
-        helpmenu.add_command(label='About', underline=0,
-                             command=self.about)
-        helpmenu.add_command(label='Instructions', underline=0,
-                             command=self.help, accelerator='F1')
+        helpmenu.add_command(label='About', underline=0, command=self.about)
+        helpmenu.add_command(
+            label='Instructions', underline=0, command=self.help, accelerator='F1'
+        )
         menubar.add_cascade(label='Help', underline=0, menu=helpmenu)
 
         parent.config(menu=menubar)
@@ -317,37 +394,52 @@ class ShiftReduceApp(object):
     def _init_feedback(self, parent):
         self._feedbackframe = feedbackframe = Frame(parent)
         feedbackframe.pack(fill='x', side='bottom', padx=3, pady=3)
-        self._lastoper_label = Label(feedbackframe, text='Last Operation:',
-                                     font=self._font)
+        self._lastoper_label = Label(
+            feedbackframe, text='Last Operation:', font=self._font
+        )
         self._lastoper_label.pack(side='left')
         lastoperframe = Frame(feedbackframe, relief='sunken', border=1)
         lastoperframe.pack(fill='x', side='right', expand=1, padx=5)
-        self._lastoper1 = Label(lastoperframe, foreground='#007070',
-                                background='#f0f0f0', font=self._font)
-        self._lastoper2 = Label(lastoperframe, anchor='w', width=30,
-                                foreground='#004040', background='#f0f0f0',
-                                font=self._font)
+        self._lastoper1 = Label(
+            lastoperframe, foreground='#007070', background='#f0f0f0', font=self._font
+        )
+        self._lastoper2 = Label(
+            lastoperframe,
+            anchor='w',
+            width=30,
+            foreground='#004040',
+            background='#f0f0f0',
+            font=self._font,
+        )
         self._lastoper1.pack(side='left')
         self._lastoper2.pack(side='left', fill='x', expand=1)
 
     def _init_canvas(self, parent):
-        self._cframe = CanvasFrame(parent, background='white',
-                                   width=525, closeenough=10,
-                                   border=2, relief='sunken')
+        self._cframe = CanvasFrame(
+            parent,
+            background='white',
+            width=525,
+            closeenough=10,
+            border=2,
+            relief='sunken',
+        )
         self._cframe.pack(expand=1, fill='both', side='top', pady=2)
         canvas = self._canvas = self._cframe.canvas()
 
         self._stackwidgets = []
         self._rtextwidgets = []
-        self._titlebar = canvas.create_rectangle(0,0,0,0, fill='#c0f0f0',
-                                                 outline='black')
-        self._exprline = canvas.create_line(0,0,0,0, dash='.')
-        self._stacktop = canvas.create_line(0,0,0,0, fill='#408080')
-        size = self._size.get()+4
-        self._stacklabel = TextWidget(canvas, 'Stack', color='#004040',
-                                      font=self._boldfont)
-        self._rtextlabel = TextWidget(canvas, 'Remaining Text',
-                                      color='#004040', font=self._boldfont)
+        self._titlebar = canvas.create_rectangle(
+            0, 0, 0, 0, fill='#c0f0f0', outline='black'
+        )
+        self._exprline = canvas.create_line(0, 0, 0, 0, dash='.')
+        self._stacktop = canvas.create_line(0, 0, 0, 0, fill='#408080')
+        size = self._size.get() + 4
+        self._stacklabel = TextWidget(
+            canvas, 'Stack', color='#004040', font=self._boldfont
+        )
+        self._rtextlabel = TextWidget(
+            canvas, 'Remaining Text', color='#004040', font=self._boldfont
+        )
         self._cframe.add_widget(self._stacklabel)
         self._cframe.add_widget(self._rtextlabel)
 
@@ -369,30 +461,32 @@ class ShiftReduceApp(object):
 
         # Position the titlebar & exprline
         (x1, y1, x2, y2) = self._stacklabel.bbox()
-        y = y2-y1+10
-        self._canvas.coords(self._titlebar, -5000, 0, 5000, y-4)
-        self._canvas.coords(self._exprline, 0, y*2-10, 5000, y*2-10)
+        y = y2 - y1 + 10
+        self._canvas.coords(self._titlebar, -5000, 0, 5000, y - 4)
+        self._canvas.coords(self._exprline, 0, y * 2 - 10, 5000, y * 2 - 10)
 
         # Position the titlebar labels..
         (x1, y1, x2, y2) = self._stacklabel.bbox()
-        self._stacklabel.move(5-x1, 3-y1)
+        self._stacklabel.move(5 - x1, 3 - y1)
         (x1, y1, x2, y2) = self._rtextlabel.bbox()
-        self._rtextlabel.move(cx2-x2-5, 3-y1)
+        self._rtextlabel.move(cx2 - x2 - 5, 3 - y1)
 
         # Draw the stack.
         stackx = 5
         for tok in self._parser.stack():
             if isinstance(tok, Tree):
-                attribs = {'tree_color': '#4080a0', 'tree_width': 2,
-                           'node_font': self._boldfont,
-                           'node_color': '#006060',
-                           'leaf_color': '#006060', 'leaf_font':self._font}
-                widget = tree_to_treesegment(self._canvas, tok,
-                                             **attribs)
+                attribs = {
+                    'tree_color': '#4080a0',
+                    'tree_width': 2,
+                    'node_font': self._boldfont,
+                    'node_color': '#006060',
+                    'leaf_color': '#006060',
+                    'leaf_font': self._font,
+                }
+                widget = tree_to_treesegment(self._canvas, tok, **attribs)
                 widget.label()['color'] = '#000000'
             else:
-                widget = TextWidget(self._canvas, tok,
-                                    color='#000000', font=self._font)
+                widget = TextWidget(self._canvas, tok, color='#000000', font=self._font)
             widget.bind_click(self._popup_reduce)
             self._stackwidgets.append(widget)
             self._cframe.add_widget(widget, stackx, y)
@@ -401,8 +495,7 @@ class ShiftReduceApp(object):
         # Draw the remaining text.
         rtextwidth = 0
         for tok in self._parser.remaining_text():
-            widget = TextWidget(self._canvas, tok,
-                                color='#000000', font=self._font)
+            widget = TextWidget(self._canvas, tok, color='#000000', font=self._font)
             self._rtextwidgets.append(widget)
             self._cframe.add_widget(widget, rtextwidth, y)
             rtextwidth = widget.bbox()[2] + 4
@@ -414,23 +507,27 @@ class ShiftReduceApp(object):
         # Move the remaining text to the correct location (keep it
         # right-justified, when possible); and move the remaining text
         # label, if necessary.
-        stackx = max(stackx, self._stacklabel.width()+25)
-        rlabelwidth = self._rtextlabel.width()+10
-        if stackx >= cx2-max(rtextwidth, rlabelwidth):
+        stackx = max(stackx, self._stacklabel.width() + 25)
+        rlabelwidth = self._rtextlabel.width() + 10
+        if stackx >= cx2 - max(rtextwidth, rlabelwidth):
             cx2 = stackx + max(rtextwidth, rlabelwidth)
         for rtextwidget in self._rtextwidgets:
-            rtextwidget.move(4+cx2-rtextwidth, 0)
-        self._rtextlabel.move(cx2-self._rtextlabel.bbox()[2]-5, 0)
+            rtextwidget.move(4 + cx2 - rtextwidth, 0)
+        self._rtextlabel.move(cx2 - self._rtextlabel.bbox()[2] - 5, 0)
 
-        midx = (stackx + cx2-max(rtextwidth, rlabelwidth))/2
+        midx = (stackx + cx2 - max(rtextwidth, rlabelwidth)) / 2
         self._canvas.coords(self._stacktop, midx, 0, midx, 5000)
         (x1, y1, x2, y2) = self._stacklabel.bbox()
 
         # Set up binding to allow them to shift a token by dragging it.
         if len(self._rtextwidgets) > 0:
+
             def drag_shift(widget, midx=midx, self=self):
-                if widget.bbox()[0] < midx: self.shift()
-                else: self._redraw()
+                if widget.bbox()[0] < midx:
+                    self.shift()
+                else:
+                    self._redraw()
+
             self._rtextwidgets[0].bind_drag(drag_shift)
             self._rtextwidgets[0].bind_click(self.shift)
 
@@ -439,7 +536,7 @@ class ShiftReduceApp(object):
 
     def _draw_stack_top(self, widget):
         # hack..
-        midx = widget.bbox()[2]+50
+        midx = widget.bbox()[2] + 50
         self._canvas.coords(self._stacktop, midx, 0, midx, 5000)
 
     def _highlight_productions(self):
@@ -454,7 +551,8 @@ class ShiftReduceApp(object):
     #########################################
 
     def destroy(self, *e):
-        if self._top is None: return
+        if self._top is None:
+            return
         self._top.destroy()
         self._top = None
 
@@ -465,8 +563,10 @@ class ShiftReduceApp(object):
         self._redraw()
 
     def step(self, *e):
-        if self.reduce(): return True
-        elif self.shift(): return True
+        if self.reduce():
+            return True
+        elif self.shift():
+            return True
         else:
             if list(self._parser.parses()):
                 self._lastoper1['text'] = 'Finished:'
@@ -476,7 +576,8 @@ class ShiftReduceApp(object):
                 self._lastoper2['text'] = 'Failure'
 
     def shift(self, *e):
-        if self._animating_lock: return
+        if self._animating_lock:
+            return
         if self._parser.shift():
             tok = self._parser.stack()[-1]
             self._lastoper1['text'] = 'Shift:'
@@ -489,7 +590,8 @@ class ShiftReduceApp(object):
         return False
 
     def reduce(self, *e):
-        if self._animating_lock: return
+        if self._animating_lock:
+            return
         production = self._parser.reduce()
         if production:
             self._lastoper1['text'] = 'Reduce:'
@@ -501,7 +603,8 @@ class ShiftReduceApp(object):
         return production
 
     def undo(self, *e):
-        if self._animating_lock: return
+        if self._animating_lock:
+            return
         if self._parser.undo():
             self._redraw()
 
@@ -515,7 +618,8 @@ class ShiftReduceApp(object):
         from a secript); otherwise, the demo will close as soon as
         the script completes.
         """
-        if in_idle(): return
+        if in_idle():
+            return
         self._top.mainloop(*args, **kwargs)
 
     #########################################
@@ -523,36 +627,46 @@ class ShiftReduceApp(object):
     #########################################
 
     def resize(self, size=None):
-        if size is not None: self._size.set(size)
+        if size is not None:
+            self._size.set(size)
         size = self._size.get()
         self._font.configure(size=-(abs(size)))
         self._boldfont.configure(size=-(abs(size)))
         self._sysfont.configure(size=-(abs(size)))
 
-        #self._stacklabel['font'] = ('helvetica', -size-4, 'bold')
-        #self._rtextlabel['font'] = ('helvetica', -size-4, 'bold')
-        #self._lastoper_label['font'] = ('helvetica', -size)
-        #self._lastoper1['font'] = ('helvetica', -size)
-        #self._lastoper2['font'] = ('helvetica', -size)
-        #self._prodlist['font'] = ('helvetica', -size)
-        #self._prodlist_label['font'] = ('helvetica', -size-2, 'bold')
+        # self._stacklabel['font'] = ('helvetica', -size-4, 'bold')
+        # self._rtextlabel['font'] = ('helvetica', -size-4, 'bold')
+        # self._lastoper_label['font'] = ('helvetica', -size)
+        # self._lastoper1['font'] = ('helvetica', -size)
+        # self._lastoper2['font'] = ('helvetica', -size)
+        # self._prodlist['font'] = ('helvetica', -size)
+        # self._prodlist_label['font'] = ('helvetica', -size-2, 'bold')
         self._redraw()
 
     def help(self, *e):
         # The default font's not very legible; try using 'fixed' instead.
         try:
-            ShowText(self._top, 'Help: Shift-Reduce Parser Application',
-                     (__doc__ or '').strip(), width=75, font='fixed')
+            ShowText(
+                self._top,
+                'Help: Shift-Reduce Parser Application',
+                (__doc__ or '').strip(),
+                width=75,
+                font='fixed',
+            )
         except:
-            ShowText(self._top, 'Help: Shift-Reduce Parser Application',
-                     (__doc__ or '').strip(), width=75)
+            ShowText(
+                self._top,
+                'Help: Shift-Reduce Parser Application',
+                (__doc__ or '').strip(),
+                width=75,
+            )
 
     def about(self, *e):
-        ABOUT = ("NLTK Shift-Reduce Parser Application\n"+
-                 "Written by Edward Loper")
+        ABOUT = "NLTK Shift-Reduce Parser Application\n" + "Written by Edward Loper"
         TITLE = 'About: Shift-Reduce Parser Application'
         try:
-            from tkinter.messagebox import Message
+            from six.moves.tkinter_messagebox import Message
+
             Message(message=ABOUT, title=TITLE).show()
         except:
             ShowText(self._top, TITLE, ABOUT)
@@ -574,7 +688,7 @@ class ShiftReduceApp(object):
         EntryDialog(self._top, sentence, instr, self.set_sentence, title)
 
     def set_sentence(self, sent):
-        self._sent = sent.split() #[XX] use tagged?
+        self._sent = sent.split()  # [XX] use tagged?
         self.reset()
 
     #########################################
@@ -583,8 +697,9 @@ class ShiftReduceApp(object):
 
     def _toggle_grammar(self, *e):
         if self._show_grammar.get():
-            self._prodframe.pack(fill='both', side='left', padx=2,
-                                 after=self._feedbackframe)
+            self._prodframe.pack(
+                fill='both', side='left', padx=2, after=self._feedbackframe
+            )
             self._lastoper1['text'] = 'Show Grammar'
         else:
             self._prodframe.pack_forget()
@@ -593,7 +708,8 @@ class ShiftReduceApp(object):
 
     def _prodlist_select(self, event):
         selection = self._prodlist.curselection()
-        if len(selection) != 1: return
+        if len(selection) != 1:
+            return
         index = int(selection[0])
         production = self._parser.reduce(self._productions[index])
         if production:
@@ -613,14 +729,15 @@ class ShiftReduceApp(object):
     def _popup_reduce(self, widget):
         # Remove old commands.
         productions = self._parser.reducible_productions()
-        if len(productions) == 0: return
+        if len(productions) == 0:
+            return
 
         self._reduce_menu.delete(0, 'end')
         for production in productions:
-            self._reduce_menu.add_command(label=str(production),
-                                          command=self.reduce)
-        self._reduce_menu.post(self._canvas.winfo_pointerx(),
-                               self._canvas.winfo_pointery())
+            self._reduce_menu.add_command(label=str(production), command=self.reduce)
+        self._reduce_menu.post(
+            self._canvas.winfo_pointerx(), self._canvas.winfo_pointery()
+        )
 
     #########################################
     ##  Animations
@@ -632,20 +749,21 @@ class ShiftReduceApp(object):
 
         # Where are we shifting from & to?
         right = widget.bbox()[0]
-        if len(self._stackwidgets) == 0: left = 5
-        else: left = self._stackwidgets[-1].bbox()[2]+10
+        if len(self._stackwidgets) == 0:
+            left = 5
+        else:
+            left = self._stackwidgets[-1].bbox()[2] + 10
 
         # Start animating.
         dt = self._animate.get()
-        dx = (left-right)*1.0/dt
+        dx = (left - right) * 1.0 / dt
         self._animate_shift_frame(dt, widget, dx)
 
     def _animate_shift_frame(self, frame, widget, dx):
         if frame > 0:
             self._animating_lock = 1
             widget.move(dx, 0)
-            self._top.after(10, self._animate_shift_frame,
-                            frame-1, widget, dx)
+            self._top.after(10, self._animate_shift_frame, frame - 1, widget, dx)
         else:
             # but: stacktop??
 
@@ -660,7 +778,7 @@ class ShiftReduceApp(object):
 
     def _animate_reduce(self):
         # What widgets are we shifting?
-        numwidgets = len(self._parser.stack()[-1]) # number of children
+        numwidgets = len(self._parser.stack()[-1])  # number of children
         widgets = self._stackwidgets[-numwidgets:]
 
         # How far are we moving?
@@ -671,29 +789,32 @@ class ShiftReduceApp(object):
 
         # Start animating.
         dt = self._animate.get()
-        dy = ydist*2.0/dt
-        self._animate_reduce_frame(dt/2, widgets, dy)
+        dy = ydist * 2.0 / dt
+        self._animate_reduce_frame(dt / 2, widgets, dy)
 
     def _animate_reduce_frame(self, frame, widgets, dy):
         if frame > 0:
             self._animating_lock = 1
-            for widget in widgets: widget.move(0, dy)
-            self._top.after(10, self._animate_reduce_frame,
-                            frame-1, widgets, dy)
+            for widget in widgets:
+                widget.move(0, dy)
+            self._top.after(10, self._animate_reduce_frame, frame - 1, widgets, dy)
         else:
-            del self._stackwidgets[-len(widgets):]
+            del self._stackwidgets[-len(widgets) :]
             for widget in widgets:
                 self._cframe.remove_widget(widget)
             tok = self._parser.stack()[-1]
-            if not isinstance(tok, Tree): raise ValueError()
-            label = TextWidget(self._canvas, str(tok.label()), color='#006060',
-                               font=self._boldfont)
-            widget = TreeSegmentWidget(self._canvas, label, widgets,
-                                       width=2)
+            if not isinstance(tok, Tree):
+                raise ValueError()
+            label = TextWidget(
+                self._canvas, str(tok.label()), color='#006060', font=self._boldfont
+            )
+            widget = TreeSegmentWidget(self._canvas, label, widgets, width=2)
             (x1, y1, x2, y2) = self._stacklabel.bbox()
-            y = y2-y1+10
-            if not self._stackwidgets: x = 5
-            else: x = self._stackwidgets[-1].bbox()[2] + 10
+            y = y2 - y1 + 10
+            if not self._stackwidgets:
+                x = 5
+            else:
+                x = self._stackwidgets[-1].bbox()[2] + 10
             self._cframe.add_widget(widget, x, y)
             self._stackwidgets.append(widget)
 
@@ -701,32 +822,32 @@ class ShiftReduceApp(object):
             self._draw_stack_top(widget)
             self._highlight_productions()
 
-#             # Delete the old widgets..
-#             del self._stackwidgets[-len(widgets):]
-#             for widget in widgets:
-#                 self._cframe.destroy_widget(widget)
-#
-#             # Make a new one.
-#             tok = self._parser.stack()[-1]
-#             if isinstance(tok, Tree):
-#                 attribs = {'tree_color': '#4080a0', 'tree_width': 2,
-#                            'node_font': bold, 'node_color': '#006060',
-#                            'leaf_color': '#006060', 'leaf_font':self._font}
-#                 widget = tree_to_treesegment(self._canvas, tok.type(),
-#                                              **attribs)
-#                 widget.node()['color'] = '#000000'
-#             else:
-#                 widget = TextWidget(self._canvas, tok.type(),
-#                                     color='#000000', font=self._font)
-#             widget.bind_click(self._popup_reduce)
-#             (x1, y1, x2, y2) = self._stacklabel.bbox()
-#             y = y2-y1+10
-#             if not self._stackwidgets: x = 5
-#             else: x = self._stackwidgets[-1].bbox()[2] + 10
-#             self._cframe.add_widget(widget, x, y)
-#             self._stackwidgets.append(widget)
+            #             # Delete the old widgets..
+            #             del self._stackwidgets[-len(widgets):]
+            #             for widget in widgets:
+            #                 self._cframe.destroy_widget(widget)
+            #
+            #             # Make a new one.
+            #             tok = self._parser.stack()[-1]
+            #             if isinstance(tok, Tree):
+            #                 attribs = {'tree_color': '#4080a0', 'tree_width': 2,
+            #                            'node_font': bold, 'node_color': '#006060',
+            #                            'leaf_color': '#006060', 'leaf_font':self._font}
+            #                 widget = tree_to_treesegment(self._canvas, tok.type(),
+            #                                              **attribs)
+            #                 widget.node()['color'] = '#000000'
+            #             else:
+            #                 widget = TextWidget(self._canvas, tok.type(),
+            #                                     color='#000000', font=self._font)
+            #             widget.bind_click(self._popup_reduce)
+            #             (x1, y1, x2, y2) = self._stacklabel.bbox()
+            #             y = y2-y1+10
+            #             if not self._stackwidgets: x = 5
+            #             else: x = self._stackwidgets[-1].bbox()[2] + 10
+            #             self._cframe.add_widget(widget, x, y)
+            #             self._stackwidgets.append(widget)
 
-            #self._redraw()
+            # self._redraw()
             self._animating_lock = 0
 
     #########################################
@@ -736,7 +857,8 @@ class ShiftReduceApp(object):
     def _highlight_hover(self, event):
         # What production are we hovering over?
         index = self._prodlist.nearest(event.y)
-        if self._hover == index: return
+        if self._hover == index:
+            return
 
         # Clear any previous hover highlighting.
         self._clear_hover()
@@ -757,7 +879,8 @@ class ShiftReduceApp(object):
 
     def _clear_hover(self, *event):
         # Clear any previous hover highlighting.
-        if self._hover == -1: return
+        if self._hover == -1:
+            return
         self._hover = -1
         for stackwidget in self._stackwidgets:
             if isinstance(stackwidget, TreeSegmentWidget):
@@ -773,9 +896,9 @@ def app():
     """
 
     from nltk.grammar import Nonterminal, Production, CFG
+
     nonterminals = 'S VP NP PP P N Name V Det'
-    (S, VP, NP, PP, P, N, Name, V, Det) = [Nonterminal(s)
-                                           for s in nonterminals.split()]
+    (S, VP, NP, PP, P, N, Name, V, Det) = [Nonterminal(s) for s in nonterminals.split()]
 
     productions = (
         # Syntactic Productions
@@ -786,15 +909,19 @@ def app():
         Production(VP, [V, NP, PP]),
         Production(VP, [V, NP]),
         Production(PP, [P, NP]),
-
         # Lexical Productions
-        Production(NP, ['I']),   Production(Det, ['the']),
-        Production(Det, ['a']),  Production(N, ['man']),
-        Production(V, ['saw']),  Production(P, ['in']),
-        Production(P, ['with']), Production(N, ['park']),
-        Production(N, ['dog']),  Production(N, ['statue']),
+        Production(NP, ['I']),
+        Production(Det, ['the']),
+        Production(Det, ['a']),
+        Production(N, ['man']),
+        Production(V, ['saw']),
+        Production(P, ['in']),
+        Production(P, ['with']),
+        Production(N, ['park']),
+        Production(N, ['dog']),
+        Production(N, ['statue']),
         Production(Det, ['my']),
-        )
+    )
 
     grammar = CFG(S, productions)
 
@@ -803,6 +930,7 @@ def app():
 
     ShiftReduceApp(grammar, sent).mainloop()
 
+
 if __name__ == '__main__':
     app()
 
