@@ -1,6 +1,6 @@
 # Natural Language Toolkit: Evaluation
 #
-# Copyright (C) 2001-2017 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 #         Steven Bird <stevenbird1@gmail.com>
 # URL: <http://nltk.org/>
@@ -12,14 +12,16 @@ import operator
 from random import shuffle
 from functools import reduce
 
+from six.moves import range, zip
+
 try:
     from scipy.stats.stats import betai
 except ImportError:
     betai = None
 
-from nltk.compat import xrange, izip
 from nltk.util import LazyConcatenation, LazyMap
 
+
 def accuracy(reference, test):
     """
     Given a list of reference values and a corresponding list of test
@@ -37,7 +39,8 @@ def accuracy(reference, test):
     """
     if len(reference) != len(test):
         raise ValueError("Lists must have the same length.")
-    return sum(x == y for x, y in izip(reference, test)) / len(test)
+    return sum(x == y for x, y in zip(reference, test)) / len(test)
+
 
 def precision(reference, test):
     """
@@ -52,8 +55,7 @@ def precision(reference, test):
     :param test: A set of values to compare against the reference set.
     :rtype: float or None
     """
-    if (not hasattr(reference, 'intersection') or
-        not hasattr(test, 'intersection')):
+    if not hasattr(reference, 'intersection') or not hasattr(test, 'intersection'):
         raise TypeError('reference and test should be sets')
 
     if len(test) == 0:
@@ -61,6 +63,7 @@ def precision(reference, test):
     else:
         return len(reference.intersection(test)) / len(test)
 
+
 def recall(reference, test):
     """
     Given a set of reference values and a set of test values, return
@@ -74,8 +77,7 @@ def recall(reference, test):
     :param test: A set of values to compare against the reference set.
     :rtype: float or None
     """
-    if (not hasattr(reference, 'intersection') or
-        not hasattr(test, 'intersection')):
+    if not hasattr(reference, 'intersection') or not hasattr(test, 'intersection'):
         raise TypeError('reference and test should be sets')
 
     if len(reference) == 0:
@@ -83,6 +85,7 @@ def recall(reference, test):
     else:
         return len(reference.intersection(test)) / len(reference)
 
+
 def f_measure(reference, test, alpha=0.5):
     """
     Given a set of reference values and a set of test values, return
@@ -113,7 +116,8 @@ def f_measure(reference, test, alpha=0.5):
         return None
     if p == 0 or r == 0:
         return 0
-    return 1.0 / (alpha / p + (1-alpha) / r)
+    return 1.0 / (alpha / p + (1 - alpha) / r)
+
 
 def log_likelihood(reference, test):
     """
@@ -131,10 +135,10 @@ def log_likelihood(reference, test):
         raise ValueError("Lists must have the same length.")
 
     # Return the average value of dist.logprob(val).
-    total_likelihood = sum(dist.logprob(val)
-                            for (val, dist) in izip(reference, test))
+    total_likelihood = sum(dist.logprob(val) for (val, dist) in zip(reference, test))
     return total_likelihood / len(reference)
 
+
 def approxrand(a, b, **kwargs):
     """
     Returns an approximate significance level between two lists of
@@ -158,8 +162,7 @@ def approxrand(a, b, **kwargs):
     """
     shuffles = kwargs.get('shuffles', 999)
     # there's no point in trying to shuffle beyond all possible permutations
-    shuffles = \
-        min(shuffles, reduce(operator.mul, xrange(1, len(a) + len(b) + 1)))
+    shuffles = min(shuffles, reduce(operator.mul, range(1, len(a) + len(b) + 1)))
     stat = kwargs.get('statistic', lambda lst: sum(lst) / len(lst))
     verbose = kwargs.get('verbose', False)
 
@@ -176,14 +179,14 @@ def approxrand(a, b, **kwargs):
     lst = LazyConcatenation([a, b])
     indices = list(range(len(a) + len(b)))
 
-    for i in xrange(shuffles):
+    for i in range(shuffles):
         if verbose and i % 10 == 0:
             print('shuffle: %d' % i)
 
         shuffle(indices)
 
-        pseudo_stat_a = stat(LazyMap(lambda i: lst[i], indices[:len(a)]))
-        pseudo_stat_b = stat(LazyMap(lambda i: lst[i], indices[len(a):]))
+        pseudo_stat_a = stat(LazyMap(lambda i: lst[i], indices[: len(a)]))
+        pseudo_stat_b = stat(LazyMap(lambda i: lst[i], indices[len(a) :]))
         pseudo_stat = fabs(pseudo_stat_a - pseudo_stat_b)
 
         if pseudo_stat >= actual_stat:
@@ -206,14 +209,14 @@ def approxrand(a, b, **kwargs):
 
 
 def demo():
-    print('-'*75)
+    print('-' * 75)
     reference = 'DET NN VB DET JJ NN NN IN DET NN'.split()
-    test    = 'DET VB VB DET NN NN NN IN DET NN'.split()
+    test = 'DET VB VB DET NN NN NN IN DET NN'.split()
     print('Reference =', reference)
     print('Test    =', test)
     print('Accuracy:', accuracy(reference, test))
 
-    print('-'*75)
+    print('-' * 75)
     reference_set = set(reference)
     test_set = set(test)
     print('Reference =', reference_set)
@@ -221,8 +224,8 @@ def demo():
     print('Precision:', precision(reference_set, test_set))
     print('   Recall:', recall(reference_set, test_set))
     print('F-Measure:', f_measure(reference_set, test_set))
-    print('-'*75)
+    print('-' * 75)
+
 
 if __name__ == '__main__':
     demo()
-
