@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Natural Language Toolkit: Interface to the CoreNLP REST API.
 #
-# Copyright (C) 2001-2016 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Dmitrijs Milajevs <dimazest@gmail.com>
 #
 # URL: <http://nltk.org/>
@@ -22,6 +22,8 @@ from nltk.tokenize.api import TokenizerI
 from nltk.parse.dependencygraph import DependencyGraph
 from nltk.tree import Tree
 
+from unittest import skip
+
 _stanford_url = 'http://stanfordnlp.github.io/CoreNLP/'
 
 
@@ -45,30 +47,32 @@ class CoreNLPServer(object):
     _JAR = r'stanford-corenlp-(\d+)\.(\d+)\.(\d+)\.jar'
 
     def __init__(
-        self, path_to_jar=None, path_to_models_jar=None, verbose=False,
-        java_options=None, corenlp_options=None, port=None,
+        self,
+        path_to_jar=None,
+        path_to_models_jar=None,
+        verbose=False,
+        java_options=None,
+        corenlp_options=None,
+        port=None,
     ):
 
         if corenlp_options is None:
-            corenlp_options = [
-                '-preload', 'tokenize,ssplit,pos,lemma,parse,depparse',
-            ]
+            corenlp_options = ['-preload', 'tokenize,ssplit,pos,lemma,parse,depparse']
 
-        jars = list(find_jar_iter(
-            self._JAR,
-            path_to_jar,
-            env_vars=('CORENLP', ),
-            searchpath=(),
-            url=_stanford_url,
-            verbose=verbose,
-            is_regex=True,
-        ))
+        jars = list(
+            find_jar_iter(
+                self._JAR,
+                path_to_jar,
+                env_vars=('CORENLP',),
+                searchpath=(),
+                url=_stanford_url,
+                verbose=verbose,
+                is_regex=True,
+            )
+        )
 
         # find the most recent code and model jar
-        stanford_jar = max(
-            jars,
-            key=lambda model_name: re.match(self._JAR, model_name)
-        )
+        stanford_jar = max(jars, key=lambda model_name: re.match(self._JAR, model_name))
 
         if port is None:
             try:
@@ -85,13 +89,13 @@ class CoreNLPServer(object):
             find_jar_iter(
                 self._MODEL_JAR_PATTERN,
                 path_to_models_jar,
-                env_vars=('CORENLP_MODELS', ),
+                env_vars=('CORENLP_MODELS',),
                 searchpath=(),
                 url=_stanford_url,
                 verbose=verbose,
                 is_regex=True,
             ),
-            key=lambda model_name: re.match(self._MODEL_JAR_PATTERN, model_name)
+            key=lambda model_name: re.match(self._MODEL_JAR_PATTERN, model_name),
         )
 
         self.verbose = verbose
@@ -134,7 +138,7 @@ class CoreNLPServer(object):
             raise CoreNLPServerError(
                 returncode,
                 'Could not start the server. '
-                'The error was: {}'.format(stderrdata.decode('ascii'))
+                'The error was: {}'.format(stderrdata.decode('ascii')),
             )
 
         for i in range(30):
@@ -146,9 +150,7 @@ class CoreNLPServer(object):
                 if response.ok:
                     break
         else:
-            raise CoreNLPServerError(
-                'Could not connect to the server.'
-            )
+            raise CoreNLPServerError('Could not connect to the server.')
 
         for i in range(60):
             try:
@@ -159,9 +161,7 @@ class CoreNLPServer(object):
                 if response.ok:
                     break
         else:
-            raise CoreNLPServerError(
-                'The server is not ready.'
-            )
+            raise CoreNLPServerError('The server is not ready.')
 
     def stop(self):
         self.popen.terminate()
@@ -188,7 +188,7 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
 
         if tagtype not in ['pos', 'ner', None]:
             raise ValueError("tagtype must be either 'pos', 'ner' or None")
-            
+
         self.tagtype = tagtype
 
         self.session = requests.Session()
@@ -221,17 +221,12 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
         :type sentence: str
         :rtype: iter(Tree)
         """
-        default_properties = {
-            'tokenize.whitespace': 'false',
-        }
+        default_properties = {'tokenize.whitespace': 'false'}
         default_properties.update(properties or {})
 
         return next(
             self.raw_parse_sents(
-                [sentence],
-                properties=default_properties,
-                *args,
-                **kwargs
+                [sentence], properties=default_properties, *args, **kwargs
             )
         )
 
@@ -239,7 +234,7 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
         default_properties = {
             'outputFormat': 'json',
             'annotators': 'tokenize,pos,lemma,ssplit,{parser_annotator}'.format(
-                parser_annotator=self.parser_annotator,
+                parser_annotator=self.parser_annotator
             ),
         }
 
@@ -247,9 +242,7 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
 
         response = self.session.post(
             self.url,
-            params={
-                'properties': json.dumps(default_properties),
-            },
+            params={'properties': json.dumps(default_properties)},
             data=data.encode(self.encoding),
             timeout=60,
         )
@@ -259,12 +252,7 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
         return response.json()
 
     def raw_parse_sents(
-        self,
-        sentences,
-        verbose=False,
-        properties=None,
-        *args,
-        **kwargs
+        self, sentences, verbose=False, properties=None, *args, **kwargs
     ):
         """Parse multiple sentences.
 
@@ -278,7 +266,7 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
         """
         default_properties = {
             # Only splits on '\n', never inside the sentence.
-            'ssplit.ssplit.eolonly': 'true',
+            'ssplit.ssplit.eolonly': 'true'
         }
 
         default_properties.update(properties or {})
@@ -298,7 +286,6 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
             tree = self.make_tree(parsed_sent)
             yield iter([tree])
 
-
     def parse_text(self, text, *args, **kwargs):
         """Parse a piece of text.
 
@@ -332,10 +319,7 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
         ['The', 'color', 'of', 'the', 'wall', 'is', 'blue', '.']
 
         """
-        default_properties = {
-            'annotators': 'tokenize,ssplit',
-
-        }
+        default_properties = {'annotators': 'tokenize,ssplit'}
 
         default_properties.update(properties or {})
 
@@ -391,16 +375,24 @@ class GenericCoreNLPParser(ParserI, TokenizerI, TaggerI):
         :type sentences: list(str)
         :rtype: list(list(list(tuple(str, str)))
         """
-        default_properties = {'ssplit.isOneSentence': 'true',
-                              'annotators': 'tokenize,ssplit,' }
-                              
+        default_properties = {
+            'ssplit.isOneSentence': 'true',
+            'annotators': 'tokenize,ssplit,',
+        }
+
         # Supports only 'pos' or 'ner' tags.
         assert self.tagtype in ['pos', 'ner']
         default_properties['annotators'] += self.tagtype
         for sentence in sentences:
             tagged_data = self.api_call(sentence, properties=default_properties)
-            yield [[(token['word'], token[self.tagtype]) for token in tagged_sentence['tokens']]
-                    for tagged_sentence in tagged_data['sentences']]
+            yield [
+                [
+                    (token['word'], token[self.tagtype])
+                    for token in tagged_sentence['tokens']
+                ]
+                for tagged_sentence in tagged_data['sentences']
+            ]
+
 
 class CoreNLPParser(GenericCoreNLPParser):
     """
@@ -755,11 +747,12 @@ def transform(sentence):
         )
 
 
+@skip('Skipping all CoreNLP tests.')
 def setup_module(module):
     from nose import SkipTest
-    raise SkipTest('Skipping all CoreNLP tests.')
+
     global server
-    
+
     try:
         server = CoreNLPServer(port=9000)
     except LookupError as e:
@@ -775,6 +768,6 @@ def setup_module(module):
         )
 
 
+@skip('Skipping all CoreNLP tests.')
 def teardown_module(module):
-    return
     server.stop()
