@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Natural Language Toolkit: Chart Parser for Feature-Based Grammars
 #
-# Copyright (C) 2001-2018 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Rob Speer <rspeer@mit.edu>
 #         Peter Ljunglöf <peter.ljunglof@heatherleaf.se>
 # URL: <http://nltk.org/>
@@ -19,20 +19,33 @@ from nltk.compat import python_2_unicode_compatible
 from nltk.featstruct import FeatStruct, unify, TYPE, find_variables
 from nltk.sem import logic
 from nltk.tree import Tree
-from nltk.grammar import (Nonterminal, Production, CFG,
-                          FeatStructNonterminal, is_nonterminal,
-                          is_terminal)
-from nltk.parse.chart import (TreeEdge, Chart, ChartParser, EdgeI,
-                              FundamentalRule, LeafInitRule,
-                              EmptyPredictRule, BottomUpPredictRule,
-                              SingleEdgeFundamentalRule,
-                              BottomUpPredictCombineRule,
-                              CachedTopDownPredictRule,
-                              TopDownInitRule)
+from nltk.grammar import (
+    Nonterminal,
+    Production,
+    CFG,
+    FeatStructNonterminal,
+    is_nonterminal,
+    is_terminal,
+)
+from nltk.parse.chart import (
+    TreeEdge,
+    Chart,
+    ChartParser,
+    EdgeI,
+    FundamentalRule,
+    LeafInitRule,
+    EmptyPredictRule,
+    BottomUpPredictRule,
+    SingleEdgeFundamentalRule,
+    BottomUpPredictCombineRule,
+    CachedTopDownPredictRule,
+    TopDownInitRule,
+)
 
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 # Tree Edge
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
+
 
 @python_2_unicode_compatible
 class FeatureTreeEdge(TreeEdge):
@@ -47,6 +60,7 @@ class FeatureTreeEdge(TreeEdge):
     every nonterminal in the edge whose symbol implements the
     interface ``SubstituteBindingsI``.
     """
+
     def __init__(self, span, lhs, rhs, dot=0, bindings=None):
         """
         Construct a new edge.  If the edge is incomplete (i.e., if
@@ -56,7 +70,8 @@ class FeatureTreeEdge(TreeEdge):
         clear the bindings.  See ``TreeEdge`` for a description of
         the other arguments.
         """
-        if bindings is None: bindings = {}
+        if bindings is None:
+            bindings = {}
 
         # If the edge is complete, then substitute in the bindings,
         # and then throw them away.  (If we didn't throw them away, we
@@ -82,8 +97,9 @@ class FeatureTreeEdge(TreeEdge):
             ``(index,index)``; and its dot position will be ``0``.
         :rtype: TreeEdge
         """
-        return FeatureTreeEdge(span=(index, index), lhs=production.lhs(),
-                               rhs=production.rhs(), dot=0)
+        return FeatureTreeEdge(
+            span=(index, index), lhs=production.lhs(), rhs=production.rhs(), dot=0
+        )
 
     def move_dot_forward(self, new_end, bindings=None):
         """
@@ -96,12 +112,17 @@ class FeatureTreeEdge(TreeEdge):
         :param bindings: Bindings for the new edge.
         :type bindings: dict
         """
-        return FeatureTreeEdge(span=(self._span[0], new_end),
-                               lhs=self._lhs, rhs=self._rhs,
-                               dot=self._dot+1, bindings=bindings)
+        return FeatureTreeEdge(
+            span=(self._span[0], new_end),
+            lhs=self._lhs,
+            rhs=self._rhs,
+            dot=self._dot + 1,
+            bindings=bindings,
+        )
 
     def _bind(self, nt, bindings):
-        if not isinstance(nt, FeatStructNonterminal): return nt
+        if not isinstance(nt, FeatStructNonterminal):
+            return nt
         return nt.substitute_bindings(bindings)
 
     def next_with_bindings(self):
@@ -118,26 +139,31 @@ class FeatureTreeEdge(TreeEdge):
         :return: The set of variables used by this edge.
         :rtype: set(Variable)
         """
-        return find_variables([self._lhs] + list(self._rhs) +
-                              list(self._bindings.keys()) +
-                              list(self._bindings.values()),
-                              fs_class=FeatStruct)
+        return find_variables(
+            [self._lhs]
+            + list(self._rhs)
+            + list(self._bindings.keys())
+            + list(self._bindings.values()),
+            fs_class=FeatStruct,
+        )
 
     def __str__(self):
         if self.is_complete():
             return TreeEdge.__unicode__(self)
         else:
-            bindings = '{%s}' % ', '.join('%s: %r' % item for item in
-                                           sorted(self._bindings.items()))
+            bindings = '{%s}' % ', '.join(
+                '%s: %r' % item for item in sorted(self._bindings.items())
+            )
             return '%s %s' % (TreeEdge.__unicode__(self), bindings)
 
 
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 # A specialized Chart for feature grammars
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 
 # TODO: subsumes check when adding new edges
 
+
 class FeatureChart(Chart):
     """
     A Chart for feature grammars.
@@ -151,7 +177,8 @@ class FeatureChart(Chart):
         ``restrictions`` on the edges.
         """
         # If there are no restrictions, then return all edges.
-        if restrictions=={}: return iter(self._edges)
+        if restrictions == {}:
+            return iter(self._edges)
 
         # Find the index corresponding to the given restrictions.
         restr_keys = sorted(restrictions.keys())
@@ -161,8 +188,9 @@ class FeatureChart(Chart):
         if restr_keys not in self._indexes:
             self._add_index(restr_keys)
 
-        vals = tuple(self._get_type_if_possible(restrictions[key])
-                     for key in restr_keys)
+        vals = tuple(
+            self._get_type_if_possible(restrictions[key]) for key in restr_keys
+        )
         return iter(self._indexes[restr_keys].get(vals, []))
 
     def _add_index(self, restr_keys):
@@ -180,8 +208,9 @@ class FeatureChart(Chart):
 
         # Add all existing edges to the index.
         for edge in self._edges:
-            vals = tuple(self._get_type_if_possible(getattr(edge, key)())
-                         for key in restr_keys)
+            vals = tuple(
+                self._get_type_if_possible(getattr(edge, key)()) for key in restr_keys
+            )
             index.setdefault(vals, []).append(edge)
 
     def _register_with_indexes(self, edge):
@@ -190,8 +219,9 @@ class FeatureChart(Chart):
         edge with all existing indexes.
         """
         for (restr_keys, index) in self._indexes.items():
-            vals = tuple(self._get_type_if_possible(getattr(edge, key)())
-                         for key in restr_keys)
+            vals = tuple(
+                self._get_type_if_possible(getattr(edge, key)()) for key in restr_keys
+            )
             index.setdefault(vals, []).append(edge)
 
     def _get_type_if_possible(self, item):
@@ -206,17 +236,19 @@ class FeatureChart(Chart):
 
     def parses(self, start, tree_class=Tree):
         for edge in self.select(start=0, end=self._num_leaves):
-            if ((isinstance(edge, FeatureTreeEdge)) and
-                (edge.lhs()[TYPE] == start[TYPE]) and
-                (unify(edge.lhs(), start, rename_vars=True))
-                ):
+            if (
+                (isinstance(edge, FeatureTreeEdge))
+                and (edge.lhs()[TYPE] == start[TYPE])
+                and (unify(edge.lhs(), start, rename_vars=True))
+            ):
                 for tree in self.trees(edge, complete=True, tree_class=tree_class):
                     yield tree
 
 
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 # Fundamental Rule
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
+
 
 class FeatureFundamentalRule(FundamentalRule):
     """
@@ -239,18 +271,23 @@ class FeatureFundamentalRule(FundamentalRule):
 
     assuming that B1 and B2 can be unified to generate B3.
     """
+
     def apply(self, chart, grammar, left_edge, right_edge):
         # Make sure the rule is applicable.
-        if not (left_edge.end() == right_edge.start() and
-                left_edge.is_incomplete() and
-                right_edge.is_complete() and
-                isinstance(left_edge, FeatureTreeEdge)):
+        if not (
+            left_edge.end() == right_edge.start()
+            and left_edge.is_incomplete()
+            and right_edge.is_complete()
+            and isinstance(left_edge, FeatureTreeEdge)
+        ):
             return
         found = right_edge.lhs()
         nextsym = left_edge.nextsym()
         if isinstance(right_edge, FeatureTreeEdge):
-            if not is_nonterminal(nextsym): return
-            if left_edge.nextsym()[TYPE] != right_edge.lhs()[TYPE]: return
+            if not is_nonterminal(nextsym):
+                return
+            if left_edge.nextsym()[TYPE] != right_edge.lhs()[TYPE]:
+                return
             # Create a copy of the bindings.
             bindings = left_edge.bindings()
             # We rename vars here, because we don't want variables
@@ -259,9 +296,11 @@ class FeatureFundamentalRule(FundamentalRule):
             # Unify B1 (left_edge.nextsym) with B2 (right_edge.lhs) to
             # generate B3 (result).
             result = unify(nextsym, found, bindings, rename_vars=False)
-            if result is None: return
+            if result is None:
+                return
         else:
-            if nextsym != found: return
+            if nextsym != found:
+                return
             # Create a copy of the bindings.
             bindings = left_edge.bindings()
 
@@ -272,6 +311,7 @@ class FeatureFundamentalRule(FundamentalRule):
         if chart.insert_with_backpointer(new_edge, left_edge, right_edge):
             yield new_edge
 
+
 class FeatureSingleEdgeFundamentalRule(SingleEdgeFundamentalRule):
     """
     A specialized version of the completer / single edge fundamental rule
@@ -279,28 +319,30 @@ class FeatureSingleEdgeFundamentalRule(SingleEdgeFundamentalRule):
     Rather than simply comparing the nonterminals for equality, they are
     unified.
     """
+
     _fundamental_rule = FeatureFundamentalRule()
 
     def _apply_complete(self, chart, grammar, right_edge):
         fr = self._fundamental_rule
-        for left_edge in chart.select(end=right_edge.start(),
-                                      is_complete=False,
-                                      nextsym=right_edge.lhs()):
+        for left_edge in chart.select(
+            end=right_edge.start(), is_complete=False, nextsym=right_edge.lhs()
+        ):
             for new_edge in fr.apply(chart, grammar, left_edge, right_edge):
                 yield new_edge
 
     def _apply_incomplete(self, chart, grammar, left_edge):
         fr = self._fundamental_rule
-        for right_edge in chart.select(start=left_edge.end(),
-                                       is_complete=True,
-                                       lhs=left_edge.nextsym()):
+        for right_edge in chart.select(
+            start=left_edge.end(), is_complete=True, lhs=left_edge.nextsym()
+        ):
             for new_edge in fr.apply(chart, grammar, left_edge, right_edge):
                 yield new_edge
 
 
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 # Top-Down Prediction
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
+
 
 class FeatureTopDownInitRule(TopDownInitRule):
     def apply(self, chart, grammar):
@@ -309,6 +351,7 @@ class FeatureTopDownInitRule(TopDownInitRule):
             if chart.insert(new_edge, ()):
                 yield new_edge
 
+
 class FeatureTopDownPredictRule(CachedTopDownPredictRule):
     """
     A specialized version of the (cached) top down predict rule that operates
@@ -327,10 +370,13 @@ class FeatureTopDownPredictRule(CachedTopDownPredictRule):
     for each grammar production ``B2 -> gamma``, assuming that B1
     and B2 can be unified.
     """
+
     def apply(self, chart, grammar, edge):
-        if edge.is_complete(): return
+        if edge.is_complete():
+            return
         nextsym, index = edge.nextsym(), edge.end()
-        if not is_nonterminal(nextsym): return
+        if not is_nonterminal(nextsym):
+            return
 
         # If we've already applied this rule to an edge with the same
         # next & end, and the chart & grammar have not changed, then
@@ -346,8 +392,10 @@ class FeatureTopDownPredictRule(CachedTopDownPredictRule):
             if prod.rhs():
                 first = prod.rhs()[0]
                 if is_terminal(first):
-                    if index >= chart.num_leaves(): continue
-                    if first != chart.leaf(index): continue
+                    if index >= chart.num_leaves():
+                        continue
+                    if first != chart.leaf(index):
+                        continue
 
             # We rename vars here, because we don't want variables
             # from the two different productions to match.
@@ -360,46 +408,56 @@ class FeatureTopDownPredictRule(CachedTopDownPredictRule):
         self._done[nextsym_with_bindings, index] = (chart, grammar)
 
 
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 # Bottom-Up Prediction
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
+
 
 class FeatureBottomUpPredictRule(BottomUpPredictRule):
     def apply(self, chart, grammar, edge):
-        if edge.is_incomplete(): return
+        if edge.is_incomplete():
+            return
         for prod in grammar.productions(rhs=edge.lhs()):
             if isinstance(edge, FeatureTreeEdge):
                 _next = prod.rhs()[0]
-                if not is_nonterminal(_next): continue
+                if not is_nonterminal(_next):
+                    continue
 
             new_edge = FeatureTreeEdge.from_production(prod, edge.start())
             if chart.insert(new_edge, ()):
                 yield new_edge
 
+
 class FeatureBottomUpPredictCombineRule(BottomUpPredictCombineRule):
     def apply(self, chart, grammar, edge):
-        if edge.is_incomplete(): return
+        if edge.is_incomplete():
+            return
         found = edge.lhs()
         for prod in grammar.productions(rhs=found):
             bindings = {}
             if isinstance(edge, FeatureTreeEdge):
                 _next = prod.rhs()[0]
-                if not is_nonterminal(_next): continue
+                if not is_nonterminal(_next):
+                    continue
 
                 # We rename vars here, because we don't want variables
                 # from the two different productions to match.
-                used_vars = find_variables((prod.lhs(),) + prod.rhs(),
-                                           fs_class=FeatStruct)
+                used_vars = find_variables(
+                    (prod.lhs(),) + prod.rhs(), fs_class=FeatStruct
+                )
                 found = found.rename_variables(used_vars=used_vars)
 
                 result = unify(_next, found, bindings, rename_vars=False)
-                if result is None: continue
+                if result is None:
+                    continue
 
-            new_edge = (FeatureTreeEdge.from_production(prod, edge.start())
-                        .move_dot_forward(edge.end(), bindings))
+            new_edge = FeatureTreeEdge.from_production(
+                prod, edge.start()
+            ).move_dot_forward(edge.end(), bindings)
             if chart.insert(new_edge, (edge,)):
                 yield new_edge
 
+
 class FeatureEmptyPredictRule(EmptyPredictRule):
     def apply(self, chart, grammar):
         for prod in grammar.productions(empty=True):
@@ -409,51 +467,70 @@ class FeatureEmptyPredictRule(EmptyPredictRule):
                     yield new_edge
 
 
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 # Feature Chart Parser
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
+
+TD_FEATURE_STRATEGY = [
+    LeafInitRule(),
+    FeatureTopDownInitRule(),
+    FeatureTopDownPredictRule(),
+    FeatureSingleEdgeFundamentalRule(),
+]
+BU_FEATURE_STRATEGY = [
+    LeafInitRule(),
+    FeatureEmptyPredictRule(),
+    FeatureBottomUpPredictRule(),
+    FeatureSingleEdgeFundamentalRule(),
+]
+BU_LC_FEATURE_STRATEGY = [
+    LeafInitRule(),
+    FeatureEmptyPredictRule(),
+    FeatureBottomUpPredictCombineRule(),
+    FeatureSingleEdgeFundamentalRule(),
+]
 
-TD_FEATURE_STRATEGY = [LeafInitRule(),
-                       FeatureTopDownInitRule(),
-                       FeatureTopDownPredictRule(),
-                       FeatureSingleEdgeFundamentalRule()]
-BU_FEATURE_STRATEGY = [LeafInitRule(),
-                       FeatureEmptyPredictRule(),
-                       FeatureBottomUpPredictRule(),
-                       FeatureSingleEdgeFundamentalRule()]
-BU_LC_FEATURE_STRATEGY = [LeafInitRule(),
-                          FeatureEmptyPredictRule(),
-                          FeatureBottomUpPredictCombineRule(),
-                          FeatureSingleEdgeFundamentalRule()]
 
 class FeatureChartParser(ChartParser):
-    def __init__(self, grammar,
-                 strategy=BU_LC_FEATURE_STRATEGY,
-                 trace_chart_width=20,
-                 chart_class=FeatureChart,
-                 **parser_args):
-        ChartParser.__init__(self, grammar,
-                             strategy=strategy,
-                             trace_chart_width=trace_chart_width,
-                             chart_class=chart_class,
-                             **parser_args)
+    def __init__(
+        self,
+        grammar,
+        strategy=BU_LC_FEATURE_STRATEGY,
+        trace_chart_width=20,
+        chart_class=FeatureChart,
+        **parser_args
+    ):
+        ChartParser.__init__(
+            self,
+            grammar,
+            strategy=strategy,
+            trace_chart_width=trace_chart_width,
+            chart_class=chart_class,
+            **parser_args
+        )
+
 
 class FeatureTopDownChartParser(FeatureChartParser):
     def __init__(self, grammar, **parser_args):
         FeatureChartParser.__init__(self, grammar, TD_FEATURE_STRATEGY, **parser_args)
 
+
 class FeatureBottomUpChartParser(FeatureChartParser):
     def __init__(self, grammar, **parser_args):
         FeatureChartParser.__init__(self, grammar, BU_FEATURE_STRATEGY, **parser_args)
 
+
 class FeatureBottomUpLeftCornerChartParser(FeatureChartParser):
     def __init__(self, grammar, **parser_args):
-        FeatureChartParser.__init__(self, grammar, BU_LC_FEATURE_STRATEGY, **parser_args)
+        FeatureChartParser.__init__(
+            self, grammar, BU_LC_FEATURE_STRATEGY, **parser_args
+        )
 
 
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 # Instantiate Variable Chart
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
+
 
 class InstantiateVarsChart(FeatureChart):
     """
@@ -463,6 +540,7 @@ class InstantiateVarsChart(FeatureChart):
     variables in the edge's ``lhs`` whose names start with '@' will be
     replaced by unique new ``Variable``s.
     """
+
     def __init__(self, tokens):
         FeatureChart.__init__(self, tokens)
 
@@ -471,7 +549,8 @@ class InstantiateVarsChart(FeatureChart):
         FeatureChart.initialize(self)
 
     def insert(self, edge, child_pointer_list):
-        if edge in self._instantiated: return False
+        if edge in self._instantiated:
+            return False
         self.instantiate_edge(edge)
         return FeatureChart.insert(self, edge, child_pointer_list)
 
@@ -487,32 +566,41 @@ class InstantiateVarsChart(FeatureChart):
         """
         # If the edge is a leaf, or is not complete, or is
         # already in the chart, then just return it as-is.
-        if not isinstance(edge, FeatureTreeEdge): return
-        if not edge.is_complete(): return
-        if edge in self._edge_to_cpls: return
+        if not isinstance(edge, FeatureTreeEdge):
+            return
+        if not edge.is_complete():
+            return
+        if edge in self._edge_to_cpls:
+            return
 
         # Get a list of variables that need to be instantiated.
         # If there are none, then return as-is.
         inst_vars = self.inst_vars(edge)
-        if not inst_vars: return
+        if not inst_vars:
+            return
 
         # Instantiate the edge!
         self._instantiated.add(edge)
         edge._lhs = edge.lhs().substitute_bindings(inst_vars)
 
     def inst_vars(self, edge):
-        return dict((var, logic.unique_variable())
-                    for var in edge.lhs().variables()
-                    if var.name.startswith('@'))
+        return dict(
+            (var, logic.unique_variable())
+            for var in edge.lhs().variables()
+            if var.name.startswith('@')
+        )
 
 
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
 # Demo
-#////////////////////////////////////////////////////////////
+# ////////////////////////////////////////////////////////////
+
 
 def demo_grammar():
     from nltk.grammar import FeatureGrammar
-    return FeatureGrammar.fromstring("""
+
+    return FeatureGrammar.fromstring(
+        """
 S  -> NP VP
 PP -> Prep NP
 NP -> NP PP
@@ -531,14 +619,21 @@ Verb -> "ate"
 Verb -> "saw"
 Prep -> "with"
 Prep -> "under"
-""")
+"""
+    )
 
-def demo(print_times=True, print_grammar=True,
-         print_trees=True, print_sentence=True,
-         trace=1,
-         parser=FeatureChartParser,
-         sent='I saw John with a dog with my cookie'):
+
+def demo(
+    print_times=True,
+    print_grammar=True,
+    print_trees=True,
+    print_sentence=True,
+    trace=1,
+    parser=FeatureChartParser,
+    sent='I saw John with a dog with my cookie',
+):
     import sys, time
+
     print()
     grammar = demo_grammar()
     if print_grammar:
@@ -555,20 +650,26 @@ def demo(print_times=True, print_grammar=True,
     if print_times:
         print("Time: %s" % (time.clock() - t))
     if print_trees:
-        for tree in trees: print(tree)
+        for tree in trees:
+            print(tree)
     else:
         print("Nr trees:", len(trees))
 
+
 def run_profile():
     import profile
+
     profile.run('for i in range(1): demo()', '/tmp/profile.out')
     import pstats
+
     p = pstats.Stats('/tmp/profile.out')
     p.strip_dirs().sort_stats('time', 'cum').print_stats(60)
     p.strip_dirs().sort_stats('cum', 'time').print_stats(60)
 
+
 if __name__ == '__main__':
     from nltk.data import load
+
     demo()
     print()
     grammar = load('grammars/book_grammars/feat0.fcfg')
