@@ -1,6 +1,6 @@
 # Natural Language Toolkit: Probabilistic Chart Parsers
 #
-# Copyright (C) 2001-2016 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 #         Steven Bird <stevenbird1@gmail.com>
 # URL: <http://nltk.org/>
@@ -38,6 +38,7 @@ from __future__ import print_function, unicode_literals
 # [XX] This might not be implemented quite right -- it would be better
 # to associate probabilities with child pointer lists.
 
+import random
 from functools import reduce
 from nltk.tree import Tree, ProbabilisticTree
 from nltk.grammar import Nonterminal, PCFG
@@ -48,7 +49,9 @@ from nltk.compat import python_2_unicode_compatible
 
 # Probabilistic edges
 class ProbabilisticLeafEdge(LeafEdge):
-    def prob(self): return 1.0
+    def prob(self):
+        return 1.0
+
 
 class ProbabilisticTreeEdge(TreeEdge):
     def __init__(self, prob, *args, **kwargs):
@@ -57,60 +60,79 @@ class ProbabilisticTreeEdge(TreeEdge):
         # two edges with different probabilities are not equal.
         self._comparison_key = (self._comparison_key, prob)
 
-    def prob(self): return self._prob
+    def prob(self):
+        return self._prob
 
     @staticmethod
     def from_production(production, index, p):
-        return ProbabilisticTreeEdge(p, (index, index), production.lhs(),
-                                     production.rhs(), 0)
+        return ProbabilisticTreeEdge(
+            p, (index, index), production.lhs(), production.rhs(), 0
+        )
+
 
 # Rules using probabilistic edges
 class ProbabilisticBottomUpInitRule(AbstractChartRule):
-    NUM_EDGES=0
+    NUM_EDGES = 0
+
     def apply(self, chart, grammar):
         for index in range(chart.num_leaves()):
             new_edge = ProbabilisticLeafEdge(chart.leaf(index), index)
             if chart.insert(new_edge, ()):
                 yield new_edge
 
+
 class ProbabilisticBottomUpPredictRule(AbstractChartRule):
-    NUM_EDGES=1
+    NUM_EDGES = 1
+
     def apply(self, chart, grammar, edge):
-        if edge.is_incomplete(): return
+        if edge.is_incomplete():
+            return
         for prod in grammar.productions():
             if edge.lhs() == prod.rhs()[0]:
-                new_edge = ProbabilisticTreeEdge.from_production(prod, edge.start(), prod.prob())
+                new_edge = ProbabilisticTreeEdge.from_production(
+                    prod, edge.start(), prod.prob()
+                )
                 if chart.insert(new_edge, ()):
                     yield new_edge
 
+
 class ProbabilisticFundamentalRule(AbstractChartRule):
-    NUM_EDGES=2
+    NUM_EDGES = 2
+
     def apply(self, chart, grammar, left_edge, right_edge):
         # Make sure the rule is applicable.
-        if not (left_edge.end() == right_edge.start() and
-                left_edge.nextsym() == right_edge.lhs() and
-                left_edge.is_incomplete() and right_edge.is_complete()):
+        if not (
+            left_edge.end() == right_edge.start()
+            and left_edge.nextsym() == right_edge.lhs()
+            and left_edge.is_incomplete()
+            and right_edge.is_complete()
+        ):
             return
 
         # Construct the new edge.
         p = left_edge.prob() * right_edge.prob()
-        new_edge = ProbabilisticTreeEdge(p,
-                            span=(left_edge.start(), right_edge.end()),
-                            lhs=left_edge.lhs(), rhs=left_edge.rhs(),
-                            dot=left_edge.dot()+1)
+        new_edge = ProbabilisticTreeEdge(
+            p,
+            span=(left_edge.start(), right_edge.end()),
+            lhs=left_edge.lhs(),
+            rhs=left_edge.rhs(),
+            dot=left_edge.dot() + 1,
+        )
 
         # Add it to the chart, with appropriate child pointers.
         changed_chart = False
         for cpl1 in chart.child_pointer_lists(left_edge):
-            if chart.insert(new_edge, cpl1+(right_edge,)):
+            if chart.insert(new_edge, cpl1 + (right_edge,)):
                 changed_chart = True
 
         # If we changed the chart, then generate the edge.
-        if changed_chart: yield new_edge
+        if changed_chart:
+            yield new_edge
+
 
 @python_2_unicode_compatible
 class SingleEdgeProbabilisticFundamentalRule(AbstractChartRule):
-    NUM_EDGES=1
+    NUM_EDGES = 1
 
     _fundamental_rule = ProbabilisticFundamentalRule()
 
@@ -118,20 +140,23 @@ class SingleEdgeProbabilisticFundamentalRule(AbstractChartRule):
         fr = self._fundamental_rule
         if edge1.is_incomplete():
             # edge1 = left_edge; edge2 = right_edge
-            for edge2 in chart.select(start=edge1.end(), is_complete=True,
-                                     lhs=edge1.nextsym()):
+            for edge2 in chart.select(
+                start=edge1.end(), is_complete=True, lhs=edge1.nextsym()
+            ):
                 for new_edge in fr.apply(chart, grammar, edge1, edge2):
                     yield new_edge
         else:
             # edge2 = left_edge; edge1 = right_edge
-            for edge2 in chart.select(end=edge1.start(), is_complete=False,
-                                      nextsym=edge1.lhs()):
+            for edge2 in chart.select(
+                end=edge1.start(), is_complete=False, nextsym=edge1.lhs()
+            ):
                 for new_edge in fr.apply(chart, grammar, edge2, edge1):
                     yield new_edge
 
     def __str__(self):
         return 'Fundamental Rule'
 
+
 class BottomUpProbabilisticChartParser(ParserI):
     """
     An abstract bottom-up parser for ``PCFG`` grammars that uses a ``Chart`` to
@@ -158,6 +183,7 @@ class BottomUpProbabilisticChartParser(ParserI):
     :ivar _trace: The level of tracing output that should be generated
         when parsing a text.
     """
+
     def __init__(self, grammar, beam_size=0, trace=0):
         """
         Create a new ``BottomUpProbabilisticChartParser``, that uses
@@ -212,8 +238,10 @@ class BottomUpProbabilisticChartParser(ParserI):
         # Initialize the chart.
         for edge in bu_init.apply(chart, grammar):
             if self._trace > 1:
-                print('  %-50s [%s]' % (chart.pretty_format_edge(edge,width=2),
-                                        edge.prob()))
+                print(
+                    '  %-50s [%s]'
+                    % (chart.pretty_format_edge(edge, width=2), edge.prob())
+                )
             queue.append(edge)
 
         while len(queue) > 0:
@@ -227,8 +255,10 @@ class BottomUpProbabilisticChartParser(ParserI):
             # Get the best edge.
             edge = queue.pop()
             if self._trace > 0:
-                print('  %-50s [%s]' % (chart.pretty_format_edge(edge,width=2),
-                                        edge.prob()))
+                print(
+                    '  %-50s [%s]'
+                    % (chart.pretty_format_edge(edge, width=2), edge.prob())
+                )
 
             # Apply BU & FR to it.
             queue.extend(bu.apply(chart, grammar, edge))
@@ -250,7 +280,8 @@ class BottomUpProbabilisticChartParser(ParserI):
         return iter(parses)
 
     def _setprob(self, tree, prod_probs):
-        if tree.prob() is not None: return
+        if tree.prob() is not None:
+            return
 
         # Get the prob of the CFG production.
         lhs = Nonterminal(tree.label())
@@ -291,12 +322,13 @@ class BottomUpProbabilisticChartParser(ParserI):
     def _prune(self, queue, chart):
         """ Discard items in the queue if the queue is longer than the beam."""
         if len(queue) > self.beam_size:
-            split = len(queue)-self.beam_size
+            split = len(queue) - self.beam_size
             if self._trace > 2:
                 for edge in queue[:split]:
-                    print('  %-50s [DISCARDED]' % chart.pretty_format_edge(edge,2))
+                    print('  %-50s [DISCARDED]' % chart.pretty_format_edge(edge, 2))
             del queue[:split]
 
+
 class InsideChartParser(BottomUpProbabilisticChartParser):
     """
     A bottom-up parser for ``PCFG`` grammars that tries edges in descending
@@ -312,6 +344,7 @@ class InsideChartParser(BottomUpProbabilisticChartParser):
     This sorting order results in a type of lowest-cost-first search
     strategy.
     """
+
     # Inherit constructor.
     def sort_queue(self, queue, chart):
         """
@@ -330,6 +363,7 @@ class InsideChartParser(BottomUpProbabilisticChartParser):
         """
         queue.sort(key=lambda edge: edge.prob())
 
+
 # Eventually, this will become some sort of inside-outside parser:
 # class InsideOutsideParser(BottomUpProbabilisticChartParser):
 #     def __init__(self, grammar, trace=0):
@@ -357,23 +391,28 @@ class InsideChartParser(BottomUpProbabilisticChartParser):
 #     def sort_queue(self, queue, chart):
 #         queue.sort(key=self._sortkey)
 
-import random
+
 class RandomChartParser(BottomUpProbabilisticChartParser):
     """
     A bottom-up parser for ``PCFG`` grammars that tries edges in random order.
     This sorting order results in a random search strategy.
     """
+
     # Inherit constructor
     def sort_queue(self, queue, chart):
-        i = random.randint(0, len(queue)-1)
+        i = random.randint(0, len(queue) - 1)
         (queue[-1], queue[i]) = (queue[i], queue[-1])
 
+
 class UnsortedChartParser(BottomUpProbabilisticChartParser):
     """
     A bottom-up parser for ``PCFG`` grammars that tries edges in whatever order.
     """
+
     # Inherit constructor
-    def sort_queue(self, queue, chart): return
+    def sort_queue(self, queue, chart):
+        return
+
 
 class LongestChartParser(BottomUpProbabilisticChartParser):
     """
@@ -381,14 +420,17 @@ class LongestChartParser(BottomUpProbabilisticChartParser):
     shorter ones.  This sorting order results in a type of best-first
     search strategy.
     """
+
     # Inherit constructor
     def sort_queue(self, queue, chart):
         queue.sort(key=lambda edge: edge.length())
 
+
 ##//////////////////////////////////////////////////////
 ##  Test Code
 ##//////////////////////////////////////////////////////
 
+
 def demo(choice=None, draw_parses=None, print_parses=None):
     """
     A demonstration of the probabilistic parsers.  The user is
@@ -401,7 +443,8 @@ def demo(choice=None, draw_parses=None, print_parses=None):
     from nltk.parse import pchart
 
     # Define two demos.  Each demo has a sentence and a grammar.
-    toy_pcfg1 = PCFG.fromstring("""
+    toy_pcfg1 = PCFG.fromstring(
+        """
     S -> NP VP [1.0]
     NP -> Det N [0.5] | NP PP [0.25] | 'John' [0.1] | 'I' [0.15]
     Det -> 'the' [0.8] | 'my' [0.2]
@@ -410,9 +453,11 @@ def demo(choice=None, draw_parses=None, print_parses=None):
     V -> 'ate' [0.35] | 'saw' [0.65]
     PP -> P NP [1.0]
     P -> 'with' [0.61] | 'under' [0.39]
-    """)
+    """
+    )
 
-    toy_pcfg2 = PCFG.fromstring("""
+    toy_pcfg2 = PCFG.fromstring(
+        """
     S    -> NP VP         [1.0]
     VP   -> V NP          [.59]
     VP   -> V             [.40]
@@ -436,21 +481,23 @@ def demo(choice=None, draw_parses=None, print_parses=None):
     Det  -> 'the'         [.41]
     Det  -> 'a'           [.31]
     Det  -> 'my'          [.28]
-    """)
+    """
+    )
 
-    demos = [('I saw John with my telescope', toy_pcfg1),
-             ('the boy saw Jack with Bob under the table with a telescope',
-              toy_pcfg2)]
+    demos = [
+        ('I saw John with my telescope', toy_pcfg1),
+        ('the boy saw Jack with Bob under the table with a telescope', toy_pcfg2),
+    ]
 
     if choice is None:
         # Ask the user which demo they want to use.
         print()
         for i in range(len(demos)):
-            print('%3s: %s' % (i+1, demos[i][0]))
+            print('%3s: %s' % (i + 1, demos[i][0]))
             print('     %r' % demos[i][1])
             print()
         print('Which demo (%d-%d)? ' % (1, len(demos)), end=' ')
-        choice = int(sys.stdin.readline().strip())-1
+        choice = int(sys.stdin.readline().strip()) - 1
     try:
         sent, grammar = demos[choice]
     except:
@@ -466,8 +513,8 @@ def demo(choice=None, draw_parses=None, print_parses=None):
         pchart.RandomChartParser(grammar),
         pchart.UnsortedChartParser(grammar),
         pchart.LongestChartParser(grammar),
-        pchart.InsideChartParser(grammar, beam_size = len(tokens)+1)   # was BeamParser
-        ]
+        pchart.InsideChartParser(grammar, beam_size=len(tokens) + 1),  # was BeamParser
+    ]
 
     # Run the parsers on the tokenized sentence.
     times = []
@@ -475,27 +522,37 @@ def demo(choice=None, draw_parses=None, print_parses=None):
     num_parses = []
     all_parses = {}
     for parser in parsers:
-        print('\ns: %s\nparser: %s\ngrammar: %s' % (sent,parser,grammar))
+        print('\ns: %s\nparser: %s\ngrammar: %s' % (sent, parser, grammar))
         parser.trace(3)
         t = time.time()
         parses = list(parser.parse(tokens))
-        times.append(time.time()-t)
-        p = (reduce(lambda a,b:a+b.prob(), parses, 0)/len(parses) if parses else 0)
+        times.append(time.time() - t)
+        p = reduce(lambda a, b: a + b.prob(), parses, 0) / len(parses) if parses else 0
         average_p.append(p)
         num_parses.append(len(parses))
-        for p in parses: all_parses[p.freeze()] = 1
+        for p in parses:
+            all_parses[p.freeze()] = 1
 
     # Print some summary statistics
     print()
     print('       Parser      Beam | Time (secs)   # Parses   Average P(parse)')
     print('------------------------+------------------------------------------')
     for i in range(len(parsers)):
-        print('%18s %4d |%11.4f%11d%19.14f' % (parsers[i].__class__.__name__,
-                                             parsers[i].beam_size,
-                                             times[i],num_parses[i],average_p[i]))
+        print(
+            '%18s %4d |%11.4f%11d%19.14f'
+            % (
+                parsers[i].__class__.__name__,
+                parsers[i].beam_size,
+                times[i],
+                num_parses[i],
+                average_p[i],
+            )
+        )
     parses = all_parses.keys()
-    if parses: p = reduce(lambda a,b:a+b.prob(), parses, 0)/len(parses)
-    else: p = 0
+    if parses:
+        p = reduce(lambda a, b: a + b.prob(), parses, 0) / len(parses)
+    else:
+        p = 0
     print('------------------------+------------------------------------------')
     print('%18s      |%11s%11d%19.14f' % ('(All Parses)', 'n/a', len(parses), p))
 
@@ -506,6 +563,7 @@ def demo(choice=None, draw_parses=None, print_parses=None):
         draw_parses = sys.stdin.readline().strip().lower().startswith('y')
     if draw_parses:
         from nltk.draw.tree import draw_trees
+
         print('  please wait...')
         draw_trees(*parses)
 
@@ -518,5 +576,6 @@ def demo(choice=None, draw_parses=None, print_parses=None):
         for parse in parses:
             print(parse)
 
+
 if __name__ == '__main__':
     demo()
