@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Natural Language Toolkit: Transformation-based learning
 #
-# Copyright (C) 2001-2017 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Marcus Uneson <marcus.uneson@gmail.com>
 #   based on previous (nltk2) version by
 #   Christopher Maloof, Edward Loper, Steven Bird
@@ -9,8 +9,11 @@
 # For license information, see  LICENSE.TXT
 
 from __future__ import division, print_function, unicode_literals
+from abc import ABCMeta, abstractmethod
+from six import add_metaclass
 
 
+@add_metaclass(ABCMeta)
 class Feature(object):
     """
     An abstract base class for Features. A Feature is a combination of
@@ -29,9 +32,6 @@ class Feature(object):
     to the classname.
 
     """
-    # !!FOR_FUTURE: when targeting python3 only, consider @abc.abstractmethod
-    # and metaclass=abc.ABCMeta rather than NotImplementedError
-    # http://julien.danjou.info/blog/2013/guide-python-static-class-abstract-methods
 
     json_tag = 'nltk.tbl.Feature'
     PROPERTY_NAME = None
@@ -79,15 +79,19 @@ class Feature(object):
         """
         self.positions = None  # to avoid warnings
         if end is None:
-            self.positions = tuple(sorted(set([int(i) for i in positions])))
-        else:                # positions was actually not a list, but only the start index
+            self.positions = tuple(sorted(set(int(i) for i in positions)))
+        else:  # positions was actually not a list, but only the start index
             try:
                 if positions > end:
                     raise TypeError
-                self.positions = tuple(range(positions, end+1))
+                self.positions = tuple(range(positions, end + 1))
             except TypeError:
                 # let any kind of erroneous spec raise ValueError
-                raise ValueError("illegal interval specification: (start={0}, end={1})".format(positions, end))
+                raise ValueError(
+                    "illegal interval specification: (start={0}, end={1})".format(
+                        positions, end
+                    )
+                )
 
         # set property name given in subclass, or otherwise name of subclass
         self.PROPERTY_NAME = self.__class__.PROPERTY_NAME or self.__class__.__name__
@@ -101,8 +105,7 @@ class Feature(object):
         return cls(positions)
 
     def __repr__(self):
-        return "%s(%r)" % (
-            self.__class__.__name__, list(self.positions))
+        return "%s(%r)" % (self.__class__.__name__, list(self.positions))
 
     @classmethod
     def expand(cls, starts, winlens, excludezero=False):
@@ -156,7 +159,7 @@ class Feature(object):
         """
         if not all(x > 0 for x in winlens):
             raise ValueError("non-positive window length in {0}".format(winlens))
-        xs = (starts[i:i+w] for w in winlens for i in range(len(starts)-w+1))
+        xs = (starts[i : i + w] for w in winlens for i in range(len(starts) - w + 1))
         return [cls(x) for x in xs if not (excludezero and 0 in x)]
 
     def issuperset(self, other):
@@ -187,7 +190,9 @@ class Feature(object):
 
 
         """
-        return self.__class__ is other.__class__ and set(self.positions) >= set(other.positions)
+        return self.__class__ is other.__class__ and set(self.positions) >= set(
+            other.positions
+        )
 
     def intersects(self, other):
         """
@@ -218,16 +223,22 @@ class Feature(object):
         :rtype: bool
         """
 
-        return bool((self.__class__ is other.__class__ and set(self.positions) & set(other.positions)))
+        return bool(
+            (
+                self.__class__ is other.__class__
+                and set(self.positions) & set(other.positions)
+            )
+        )
 
     # Rich comparisons for Features. With @functools.total_ordering (Python 2.7+),
     # it will be enough to define __lt__ and __eq__
     def __eq__(self, other):
-        return (self.__class__ is other.__class__ and self.positions == other.positions)
+        return self.__class__ is other.__class__ and self.positions == other.positions
 
     def __lt__(self, other):
         return (
-            self.__class__.__name__ < other.__class__.__name__ or
+            self.__class__.__name__ < other.__class__.__name__
+            or
             #    self.positions is a sorted tuple of ints
             self.positions < other.positions
         )
@@ -245,6 +256,7 @@ class Feature(object):
         return self < other or self == other
 
     @staticmethod
+    @abstractmethod
     def extract_property(tokens, index):
         """
         Any subclass of Feature must define static method extract_property(tokens, index)
@@ -256,6 +268,3 @@ class Feature(object):
         :return: feature value
         :rtype: any (but usually scalar)
         """
-        raise NotImplementedError
-
-
