@@ -5,10 +5,13 @@ from __future__ import absolute_import
 def setup_module(module):
     from nose import SkipTest
     import nltk.data
+
     try:
         nltk.data.find('corpora/childes/data-xml/Eng-USA-MOR/')
     except LookupError as e:
         print(e)
-        raise SkipTest("The CHILDES corpus is not found. "
-                       "It should be manually downloaded and saved/unpacked "
-                       "to [NLTK_Data_Dir]/corpora/childes/")
+        raise SkipTest(
+            "The CHILDES corpus is not found. "
+            "It should be manually downloaded and saved/unpacked "
+            "to [NLTK_Data_Dir]/corpora/childes/"
+        )
