@@ -3,7 +3,7 @@
 #
 # Natural Language Toolkit: TGrep search
 #
-# Copyright (C) 2001-2018 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Will Roberts <wildwilhelm@gmail.com>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
@@ -125,10 +125,13 @@ except ImportError:
 
 import nltk.tree
 
+
 class TgrepException(Exception):
     '''Tgrep exception type.'''
+
     pass
 
+
 def ancestors(node):
     '''
     Returns the list of all nodes dominating the given tree node.
@@ -146,6 +149,7 @@ def ancestors(node):
         current = current.parent()
     return results
 
+
 def unique_ancestors(node):
     '''
     Returns the list of all nodes dominating the given node, where
@@ -162,6 +166,7 @@ def unique_ancestors(node):
         current = current.parent()
     return results
 
+
 def _descendants(node):
     '''
     Returns the list of all nodes which are descended from the given
@@ -173,6 +178,7 @@ def _descendants(node):
         return []
     return [node[x] for x in treepos[1:]]
 
+
 def _leftmost_descendants(node):
     '''
     Returns the set of all nodes descended in some way through
@@ -184,6 +190,7 @@ def _leftmost_descendants(node):
         return []
     return [node[x] for x in treepos[1:] if all(y == 0 for y in x)]
 
+
 def _rightmost_descendants(node):
     '''
     Returns the set of all nodes descended in some way through
@@ -195,10 +202,12 @@ def _rightmost_descendants(node):
         return []
     return [node[rightmost_leaf[:i]] for i in range(1, len(rightmost_leaf) + 1)]
 
+
 def _istree(obj):
     '''Predicate to check whether `obj` is a nltk.tree.Tree.'''
     return isinstance(obj, nltk.tree.Tree)
 
+
 def _unique_descendants(node):
     '''
     Returns the list of all nodes descended from the given node, where
@@ -211,6 +220,7 @@ def _unique_descendants(node):
         results.append(current)
     return results
 
+
 def _before(node):
     '''
     Returns the set of all nodes that are before the given node.
@@ -220,8 +230,8 @@ def _before(node):
         tree = node.root()
     except AttributeError:
         return []
-    return [tree[x] for x in tree.treepositions()
-            if x[:len(pos)] < pos[:len(x)]]
+    return [tree[x] for x in tree.treepositions() if x[: len(pos)] < pos[: len(x)]]
+
 
 def _immediately_before(node):
     '''
@@ -243,11 +253,12 @@ def _immediately_before(node):
         idx -= 1
     if idx < 0:
         return []
-    pos = list(pos[:idx + 1])
+    pos = list(pos[: idx + 1])
     pos[-1] -= 1
     before = tree[pos]
     return [before] + _rightmost_descendants(before)
 
+
 def _after(node):
     '''
     Returns the set of all nodes that are after the given node.
@@ -257,8 +268,8 @@ def _after(node):
         tree = node.root()
     except AttributeError:
         return []
-    return [tree[x] for x in tree.treepositions()
-            if x[:len(pos)] > pos[:len(x)]]
+    return [tree[x] for x in tree.treepositions() if x[: len(pos)] > pos[: len(x)]]
+
 
 def _immediately_after(node):
     '''
@@ -283,17 +294,19 @@ def _immediately_after(node):
         current = current.parent()
     if idx < 0:
         return []
-    pos = list(pos[:idx + 1])
+    pos = list(pos[: idx + 1])
     pos[-1] += 1
     after = tree[pos]
     return [after] + _leftmost_descendants(after)
 
+
 def _tgrep_node_literal_value(node):
     '''
     Gets the string value of a given parse tree node, for comparison
     using the tgrep node literal predicates.
     '''
-    return (node.label() if _istree(node) else text_type(node))
+    return node.label() if _istree(node) else text_type(node)
+
 
 def _tgrep_macro_use_action(_s, _l, tokens):
     '''
@@ -302,12 +315,15 @@ def _tgrep_macro_use_action(_s, _l, tokens):
     assert len(tokens) == 1
     assert tokens[0][0] == '@'
     macro_name = tokens[0][1:]
+
     def macro_use(n, m=None, l=None):
         if m is None or macro_name not in m:
             raise TgrepException('macro {0} not defined'.format(macro_name))
         return m[macro_name](n, m, l)
+
     return macro_use
 
+
 def _tgrep_node_action(_s, _l, tokens):
     '''
     Builds a lambda function representing a predicate on a tree node
@@ -321,8 +337,7 @@ def _tgrep_node_action(_s, _l, tokens):
         # disjunctive definition of a node name
         assert list(set(tokens[1::2])) == ['|']
         # recursively call self to interpret each node name definition
-        tokens = [_tgrep_node_action(None, None, [node])
-                  for node in tokens[::2]]
+        tokens = [_tgrep_node_action(None, None, [node]) for node in tokens[::2]]
         # capture tokens and return the disjunction
         return (lambda t: lambda n, m=None, l=None: any(f(n, m, l) for f in t))(tokens)
     else:
@@ -335,19 +350,29 @@ def _tgrep_node_action(_s, _l, tokens):
         elif tokens[0].startswith('"'):
             assert tokens[0].endswith('"')
             node_lit = tokens[0][1:-1].replace('\\"', '"').replace('\\\\', '\\')
-            return (lambda s: lambda n, m=None, l=None: _tgrep_node_literal_value(n) == s)(node_lit)
+            return (
+                lambda s: lambda n, m=None, l=None: _tgrep_node_literal_value(n) == s
+            )(node_lit)
         elif tokens[0].startswith('/'):
             assert tokens[0].endswith('/')
             node_lit = tokens[0][1:-1]
-            return (lambda r: lambda n, m=None, l=None:
-                    r.search(_tgrep_node_literal_value(n)))(re.compile(node_lit))
+            return (
+                lambda r: lambda n, m=None, l=None: r.search(
+                    _tgrep_node_literal_value(n)
+                )
+            )(re.compile(node_lit))
         elif tokens[0].startswith('i@'):
             node_func = _tgrep_node_action(_s, _l, [tokens[0][2:].lower()])
-            return (lambda f: lambda n, m=None, l=None:
-                    f(_tgrep_node_literal_value(n).lower()))(node_func)
+            return (
+                lambda f: lambda n, m=None, l=None: f(
+                    _tgrep_node_literal_value(n).lower()
+                )
+            )(node_func)
         else:
-            return (lambda s: lambda n, m=None, l=None:
-                    _tgrep_node_literal_value(n) == s)(tokens[0])
+            return (
+                lambda s: lambda n, m=None, l=None: _tgrep_node_literal_value(n) == s
+            )(tokens[0])
+
 
 def _tgrep_parens_action(_s, _l, tokens):
     '''
@@ -360,6 +385,7 @@ def _tgrep_parens_action(_s, _l, tokens):
     assert tokens[2] == ')'
     return tokens[1]
 
+
 def _tgrep_nltk_tree_pos_action(_s, _l, tokens):
     '''
     Builds a lambda function representing a predicate on a tree node
@@ -369,8 +395,12 @@ def _tgrep_nltk_tree_pos_action(_s, _l, tokens):
     # recover the tuple from the parsed sting
     node_tree_position = tuple(int(x) for x in tokens if x.isdigit())
     # capture the node's tree position
-    return (lambda i: lambda n, m=None, l=None: (hasattr(n, 'treeposition') and
-                                                 n.treeposition() == i))(node_tree_position)
+    return (
+        lambda i: lambda n, m=None, l=None: (
+            hasattr(n, 'treeposition') and n.treeposition() == i
+        )
+    )(node_tree_position)
+
 
 def _tgrep_relation_action(_s, _l, tokens):
     '''
@@ -394,171 +424,219 @@ def _tgrep_relation_action(_s, _l, tokens):
         operator, predicate = tokens
         # A < B       A is the parent of (immediately dominates) B.
         if operator == '<':
-            retval = lambda n, m=None, l=None: (_istree(n) and
-                                                any(predicate(x, m, l) for x in n))
+            retval = lambda n, m=None, l=None: (
+                _istree(n) and any(predicate(x, m, l) for x in n)
+            )
         # A > B       A is the child of B.
         elif operator == '>':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'parent') and
-                                                bool(n.parent()) and
-                                                predicate(n.parent(), m, l))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'parent')
+                and bool(n.parent())
+                and predicate(n.parent(), m, l)
+            )
         # A <, B      Synonymous with A <1 B.
         elif operator == '<,' or operator == '<1':
-            retval = lambda n, m=None, l=None: (_istree(n) and
-                                                bool(list(n)) and
-                                                predicate(n[0], m, l))
+            retval = lambda n, m=None, l=None: (
+                _istree(n) and bool(list(n)) and predicate(n[0], m, l)
+            )
         # A >, B      Synonymous with A >1 B.
         elif operator == '>,' or operator == '>1':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'parent') and
-                                                bool(n.parent()) and
-                                                (n is n.parent()[0]) and
-                                                predicate(n.parent(), m, l))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'parent')
+                and bool(n.parent())
+                and (n is n.parent()[0])
+                and predicate(n.parent(), m, l)
+            )
         # A <N B      B is the Nth child of A (the first child is <1).
         elif operator[0] == '<' and operator[1:].isdigit():
             idx = int(operator[1:])
             # capture the index parameter
-            retval = (lambda i: lambda n, m=None, l=None: (_istree(n) and
-                                                           bool(list(n)) and
-                                                           0 <= i < len(n) and
-                                                           predicate(n[i], m, l)))(idx - 1)
+            retval = (
+                lambda i: lambda n, m=None, l=None: (
+                    _istree(n)
+                    and bool(list(n))
+                    and 0 <= i < len(n)
+                    and predicate(n[i], m, l)
+                )
+            )(idx - 1)
         # A >N B      A is the Nth child of B (the first child is >1).
         elif operator[0] == '>' and operator[1:].isdigit():
             idx = int(operator[1:])
             # capture the index parameter
-            retval = (lambda i: lambda n, m=None, l=None: (hasattr(n, 'parent') and
-                                                           bool(n.parent()) and
-                                                           0 <= i < len(n.parent()) and
-                                                           (n is n.parent()[i]) and
-                                                           predicate(n.parent(), m, l)))(idx - 1)
+            retval = (
+                lambda i: lambda n, m=None, l=None: (
+                    hasattr(n, 'parent')
+                    and bool(n.parent())
+                    and 0 <= i < len(n.parent())
+                    and (n is n.parent()[i])
+                    and predicate(n.parent(), m, l)
+                )
+            )(idx - 1)
         # A <' B      B is the last child of A (also synonymous with A <-1 B).
         # A <- B      B is the last child of A (synonymous with A <-1 B).
         elif operator == '<\'' or operator == '<-' or operator == '<-1':
-            retval = lambda n, m=None, l=None: (_istree(n) and bool(list(n))
-                                                and predicate(n[-1], m, l))
+            retval = lambda n, m=None, l=None: (
+                _istree(n) and bool(list(n)) and predicate(n[-1], m, l)
+            )
         # A >' B      A is the last child of B (also synonymous with A >-1 B).
         # A >- B      A is the last child of B (synonymous with A >-1 B).
         elif operator == '>\'' or operator == '>-' or operator == '>-1':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'parent') and
-                                                bool(n.parent()) and
-                                                (n is n.parent()[-1]) and
-                                                predicate(n.parent(), m, l))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'parent')
+                and bool(n.parent())
+                and (n is n.parent()[-1])
+                and predicate(n.parent(), m, l)
+            )
         # A <-N B 	  B is the N th-to-last child of A (the last child is <-1).
         elif operator[:2] == '<-' and operator[2:].isdigit():
             idx = -int(operator[2:])
             # capture the index parameter
-            retval = (lambda i: lambda n, m=None, l=None: (_istree(n) and
-                                                           bool(list(n)) and
-                                                           0 <= (i + len(n)) < len(n) and
-                                                           predicate(n[i + len(n)], m, l)))(idx)
+            retval = (
+                lambda i: lambda n, m=None, l=None: (
+                    _istree(n)
+                    and bool(list(n))
+                    and 0 <= (i + len(n)) < len(n)
+                    and predicate(n[i + len(n)], m, l)
+                )
+            )(idx)
         # A >-N B 	  A is the N th-to-last child of B (the last child is >-1).
         elif operator[:2] == '>-' and operator[2:].isdigit():
             idx = -int(operator[2:])
             # capture the index parameter
-            retval = (lambda i: lambda n, m=None, l=None:
-                          (hasattr(n, 'parent') and
-                           bool(n.parent()) and
-                           0 <= (i + len(n.parent())) < len(n.parent()) and
-                           (n is n.parent()[i + len(n.parent())]) and
-                           predicate(n.parent(), m, l)))(idx)
+            retval = (
+                lambda i: lambda n, m=None, l=None: (
+                    hasattr(n, 'parent')
+                    and bool(n.parent())
+                    and 0 <= (i + len(n.parent())) < len(n.parent())
+                    and (n is n.parent()[i + len(n.parent())])
+                    and predicate(n.parent(), m, l)
+                )
+            )(idx)
         # A <: B      B is the only child of A
         elif operator == '<:':
-            retval = lambda n, m=None, l=None: (_istree(n) and
-                                                len(n) == 1 and
-                                                predicate(n[0], m, l))
+            retval = lambda n, m=None, l=None: (
+                _istree(n) and len(n) == 1 and predicate(n[0], m, l)
+            )
         # A >: B      A is the only child of B.
         elif operator == '>:':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'parent') and
-                                                bool(n.parent()) and
-                                                len(n.parent()) == 1 and
-                                                predicate(n.parent(), m, l))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'parent')
+                and bool(n.parent())
+                and len(n.parent()) == 1
+                and predicate(n.parent(), m, l)
+            )
         # A << B      A dominates B (A is an ancestor of B).
         elif operator == '<<':
-            retval = lambda n, m=None, l=None: (_istree(n) and
-                                                any(predicate(x, m, l) for x in _descendants(n)))
+            retval = lambda n, m=None, l=None: (
+                _istree(n) and any(predicate(x, m, l) for x in _descendants(n))
+            )
         # A >> B      A is dominated by B (A is a descendant of B).
         elif operator == '>>':
-            retval = lambda n, m=None, l=None: any(predicate(x, m, l) for x in ancestors(n))
+            retval = lambda n, m=None, l=None: any(
+                predicate(x, m, l) for x in ancestors(n)
+            )
         # A <<, B     B is a left-most descendant of A.
         elif operator == '<<,' or operator == '<<1':
-            retval = lambda n, m=None, l=None: (_istree(n) and
-                                                any(predicate(x, m, l)
-                                                    for x in _leftmost_descendants(n)))
+            retval = lambda n, m=None, l=None: (
+                _istree(n) and any(predicate(x, m, l) for x in _leftmost_descendants(n))
+            )
         # A >>, B     A is a left-most descendant of B.
         elif operator == '>>,':
-            retval = lambda n, m=None, l=None: any((predicate(x, m, l) and
-                                                    n in _leftmost_descendants(x))
-                                                   for x in ancestors(n))
+            retval = lambda n, m=None, l=None: any(
+                (predicate(x, m, l) and n in _leftmost_descendants(x))
+                for x in ancestors(n)
+            )
         # A <<' B     B is a right-most descendant of A.
         elif operator == '<<\'':
-            retval = lambda n, m=None, l=None: (_istree(n) and
-                                                any(predicate(x, m, l)
-                                                    for x in _rightmost_descendants(n)))
+            retval = lambda n, m=None, l=None: (
+                _istree(n)
+                and any(predicate(x, m, l) for x in _rightmost_descendants(n))
+            )
         # A >>' B     A is a right-most descendant of B.
         elif operator == '>>\'':
-            retval = lambda n, m=None, l=None: any((predicate(x, m, l) and
-                                                    n in _rightmost_descendants(x))
-                                                   for x in ancestors(n))
+            retval = lambda n, m=None, l=None: any(
+                (predicate(x, m, l) and n in _rightmost_descendants(x))
+                for x in ancestors(n)
+            )
         # A <<: B     There is a single path of descent from A and B is on it.
         elif operator == '<<:':
-            retval = lambda n, m=None, l=None: (_istree(n) and
-                                                any(predicate(x, m, l)
-                                                    for x in _unique_descendants(n)))
+            retval = lambda n, m=None, l=None: (
+                _istree(n) and any(predicate(x, m, l) for x in _unique_descendants(n))
+            )
         # A >>: B     There is a single path of descent from B and A is on it.
         elif operator == '>>:':
-            retval = lambda n, m=None, l=None: any(predicate(x, m, l) for x in unique_ancestors(n))
+            retval = lambda n, m=None, l=None: any(
+                predicate(x, m, l) for x in unique_ancestors(n)
+            )
         # A . B       A immediately precedes B.
         elif operator == '.':
-            retval = lambda n, m=None, l=None: any(predicate(x, m, l)
-                                                   for x in _immediately_after(n))
+            retval = lambda n, m=None, l=None: any(
+                predicate(x, m, l) for x in _immediately_after(n)
+            )
         # A , B       A immediately follows B.
         elif operator == ',':
-            retval = lambda n, m=None, l=None: any(predicate(x, m, l)
-                                                   for x in _immediately_before(n))
+            retval = lambda n, m=None, l=None: any(
+                predicate(x, m, l) for x in _immediately_before(n)
+            )
         # A .. B      A precedes B.
         elif operator == '..':
-            retval = lambda n, m=None, l=None: any(predicate(x, m, l) for x in _after(n))
+            retval = lambda n, m=None, l=None: any(
+                predicate(x, m, l) for x in _after(n)
+            )
         # A ,, B      A follows B.
         elif operator == ',,':
-            retval = lambda n, m=None, l=None: any(predicate(x, m, l) for x in _before(n))
+            retval = lambda n, m=None, l=None: any(
+                predicate(x, m, l) for x in _before(n)
+            )
         # A $ B       A is a sister of B (and A != B).
         elif operator == '$' or operator == '%':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'parent') and
-                                                bool(n.parent()) and
-                                                any(predicate(x, m, l)
-                                                    for x in n.parent() if x is not n))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'parent')
+                and bool(n.parent())
+                and any(predicate(x, m, l) for x in n.parent() if x is not n)
+            )
         # A $. B      A is a sister of and immediately precedes B.
         elif operator == '$.' or operator == '%.':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'right_sibling') and
-                                                bool(n.right_sibling()) and
-                                                predicate(n.right_sibling(), m, l))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'right_sibling')
+                and bool(n.right_sibling())
+                and predicate(n.right_sibling(), m, l)
+            )
         # A $, B      A is a sister of and immediately follows B.
         elif operator == '$,' or operator == '%,':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'left_sibling') and
-                                                bool(n.left_sibling()) and
-                                                predicate(n.left_sibling(), m, l))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'left_sibling')
+                and bool(n.left_sibling())
+                and predicate(n.left_sibling(), m, l)
+            )
         # A $.. B     A is a sister of and precedes B.
         elif operator == '$..' or operator == '%..':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'parent') and
-                                                hasattr(n, 'parent_index') and
-                                                bool(n.parent()) and
-                                                any(predicate(x, m, l) for x in
-                                                    n.parent()[n.parent_index() + 1:]))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'parent')
+                and hasattr(n, 'parent_index')
+                and bool(n.parent())
+                and any(predicate(x, m, l) for x in n.parent()[n.parent_index() + 1 :])
+            )
         # A $,, B     A is a sister of and follows B.
         elif operator == '$,,' or operator == '%,,':
-            retval = lambda n, m=None, l=None: (hasattr(n, 'parent') and
-                                                hasattr(n, 'parent_index') and
-                                                bool(n.parent()) and
-                                                any(predicate(x, m, l) for x in
-                                                    n.parent()[:n.parent_index()]))
+            retval = lambda n, m=None, l=None: (
+                hasattr(n, 'parent')
+                and hasattr(n, 'parent_index')
+                and bool(n.parent())
+                and any(predicate(x, m, l) for x in n.parent()[: n.parent_index()])
+            )
         else:
             raise TgrepException(
-                'cannot interpret tgrep operator "{0}"'.format(operator))
+                'cannot interpret tgrep operator "{0}"'.format(operator)
+            )
     # now return the built function
     if negated:
         return (lambda r: (lambda n, m=None, l=None: not r(n, m, l)))(retval)
     else:
         return retval
 
-def _tgrep_conjunction_action(_s, _l, tokens, join_char = '&'):
+
+def _tgrep_conjunction_action(_s, _l, tokens, join_char='&'):
     '''
     Builds a lambda function representing a predicate on a tree node
     from the conjunction of several other such lambda functions.
@@ -587,8 +665,12 @@ def _tgrep_conjunction_action(_s, _l, tokens, join_char = '&'):
     if len(tokens) == 1:
         return tokens[0]
     else:
-        return (lambda ts: lambda n, m=None, l=None: all(predicate(n, m, l)
-                                                         for predicate in ts))(tokens)
+        return (
+            lambda ts: lambda n, m=None, l=None: all(
+                predicate(n, m, l) for predicate in ts
+            )
+        )(tokens)
+
 
 def _tgrep_segmented_pattern_action(_s, _l, tokens):
     '''
@@ -619,17 +701,21 @@ def _tgrep_segmented_pattern_action(_s, _l, tokens):
     # tokens[1:] is an (optional) list of predicates which must all
     # hold of the bound node
     reln_preds = tokens[1:]
+
     def pattern_segment_pred(n, m=None, l=None):
         '''This predicate function ignores its node argument.'''
         # look up the bound node using its label
         if l is None or node_label not in l:
-            raise TgrepException('node_label ={0} not bound in pattern'.format(
-                node_label))
+            raise TgrepException(
+                'node_label ={0} not bound in pattern'.format(node_label)
+            )
         node = l[node_label]
         # match the relation predicates against the node
         return all(pred(node, m, l) for pred in reln_preds)
+
     return pattern_segment_pred
 
+
 def _tgrep_node_label_use_action(_s, _l, tokens):
     '''
     Returns the node label used to begin a tgrep_expr_labeled.  See
@@ -648,6 +734,7 @@ def _tgrep_node_label_use_action(_s, _l, tokens):
     assert tokens[0].startswith('=')
     return tokens[0][1:]
 
+
 def _tgrep_node_label_pred_use_action(_s, _l, tokens):
     '''
     Builds a lambda function representing a predicate on a tree node
@@ -665,16 +752,20 @@ def _tgrep_node_label_pred_use_action(_s, _l, tokens):
     assert len(tokens) == 1
     assert tokens[0].startswith('=')
     node_label = tokens[0][1:]
+
     def node_label_use_pred(n, m=None, l=None):
         # look up the bound node using its label
         if l is None or node_label not in l:
-            raise TgrepException('node_label ={0} not bound in pattern'.format(
-                node_label))
+            raise TgrepException(
+                'node_label ={0} not bound in pattern'.format(node_label)
+            )
         node = l[node_label]
         # truth means the given node is this node
         return n is node
+
     return node_label_use_pred
 
+
 def _tgrep_bind_node_label_action(_s, _l, tokens):
     '''
     Builds a lambda function representing a predicate on a tree node
@@ -696,19 +787,24 @@ def _tgrep_bind_node_label_action(_s, _l, tokens):
         assert tokens[1] == '='
         node_pred = tokens[0]
         node_label = tokens[2]
+
         def node_label_bind_pred(n, m=None, l=None):
             if node_pred(n, m, l):
                 # bind `n` into the dictionary `l`
                 if l is None:
                     raise TgrepException(
                         'cannot bind node_label {0}: label_dict is None'.format(
-                            node_label))
+                            node_label
+                        )
+                    )
                 l[node_label] = n
                 return True
             else:
                 return False
+
         return node_label_bind_pred
 
+
 def _tgrep_rel_disjunction_action(_s, _l, tokens):
     '''
     Builds a lambda function representing a predicate on a tree node
@@ -720,8 +816,10 @@ def _tgrep_rel_disjunction_action(_s, _l, tokens):
     if len(tokens) == 1:
         return tokens[0]
     elif len(tokens) == 2:
-        return (lambda a, b: lambda n, m=None, l=None:
-                a(n, m, l) or b(n, m, l))(tokens[0], tokens[1])
+        return (lambda a, b: lambda n, m=None, l=None: a(n, m, l) or b(n, m, l))(
+            tokens[0], tokens[1]
+        )
+
 
 def _macro_defn_action(_s, _l, tokens):
     '''
@@ -731,6 +829,7 @@ def _macro_defn_action(_s, _l, tokens):
     assert tokens[0] == '@'
     return {tokens[1]: tokens[2]}
 
+
 def _tgrep_exprs_action(_s, _l, tokens):
     '''
     This is the top-lebel node in a tgrep2 search string; the
@@ -758,33 +857,40 @@ def _tgrep_exprs_action(_s, _l, tokens):
         label_dict = {}
         # bind macro definitions and OR together all tgrep_exprs
         return any(predicate(n, m, label_dict) for predicate in tgrep_exprs)
+
     return top_level_pred
 
-def _build_tgrep_parser(set_parse_actions = True):
+
+def _build_tgrep_parser(set_parse_actions=True):
     '''
     Builds a pyparsing-based parser object for tokenizing and
     interpreting tgrep search strings.
     '''
-    tgrep_op = (pyparsing.Optional('!') +
-                pyparsing.Regex('[$%,.<>][%,.<>0-9-\':]*'))
-    tgrep_qstring = pyparsing.QuotedString(quoteChar='"', escChar='\\',
-                                           unquoteResults=False)
-    tgrep_node_regex = pyparsing.QuotedString(quoteChar='/', escChar='\\',
-                                              unquoteResults=False)
-    tgrep_qstring_icase = pyparsing.Regex(
-        'i@\\"(?:[^"\\n\\r\\\\]|(?:\\\\.))*\\"')
-    tgrep_node_regex_icase = pyparsing.Regex(
-        'i@\\/(?:[^/\\n\\r\\\\]|(?:\\\\.))*\\/')
+    tgrep_op = pyparsing.Optional('!') + pyparsing.Regex('[$%,.<>][%,.<>0-9-\':]*')
+    tgrep_qstring = pyparsing.QuotedString(
+        quoteChar='"', escChar='\\', unquoteResults=False
+    )
+    tgrep_node_regex = pyparsing.QuotedString(
+        quoteChar='/', escChar='\\', unquoteResults=False
+    )
+    tgrep_qstring_icase = pyparsing.Regex('i@\\"(?:[^"\\n\\r\\\\]|(?:\\\\.))*\\"')
+    tgrep_node_regex_icase = pyparsing.Regex('i@\\/(?:[^/\\n\\r\\\\]|(?:\\\\.))*\\/')
     tgrep_node_literal = pyparsing.Regex('[^][ \r\t\n;:.,&|<>()$!@%\'^=]+')
     tgrep_expr = pyparsing.Forward()
     tgrep_relations = pyparsing.Forward()
     tgrep_parens = pyparsing.Literal('(') + tgrep_expr + ')'
     tgrep_nltk_tree_pos = (
-        pyparsing.Literal('N(') +
-        pyparsing.Optional(pyparsing.Word(pyparsing.nums) + ',' +
-                           pyparsing.Optional(pyparsing.delimitedList(
-                    pyparsing.Word(pyparsing.nums), delim=',') +
-                                              pyparsing.Optional(','))) + ')')
+        pyparsing.Literal('N(')
+        + pyparsing.Optional(
+            pyparsing.Word(pyparsing.nums)
+            + ','
+            + pyparsing.Optional(
+                pyparsing.delimitedList(pyparsing.Word(pyparsing.nums), delim=',')
+                + pyparsing.Optional(',')
+            )
+        )
+        + ')'
+    )
     tgrep_node_label = pyparsing.Regex('[A-Za-z0-9]+')
     tgrep_node_label_use = pyparsing.Combine('=' + tgrep_node_label)
     # see _tgrep_segmented_pattern_action
@@ -792,42 +898,49 @@ def _build_tgrep_parser(set_parse_actions = True):
     macro_name = pyparsing.Regex('[^];:.,&|<>()[$!@%\'^=\r\t\n ]+')
     macro_name.setWhitespaceChars('')
     macro_use = pyparsing.Combine('@' + macro_name)
-    tgrep_node_expr = (tgrep_node_label_use_pred |
-                       macro_use |
-                       tgrep_nltk_tree_pos |
-                       tgrep_qstring_icase |
-                       tgrep_node_regex_icase |
-                       tgrep_qstring |
-                       tgrep_node_regex |
-                       '*' |
-                       tgrep_node_literal)
-    tgrep_node_expr2 = ((tgrep_node_expr +
-                         pyparsing.Literal('=').setWhitespaceChars('') +
-                         tgrep_node_label.copy().setWhitespaceChars('')) |
-                        tgrep_node_expr)
-    tgrep_node = (tgrep_parens |
-                  (pyparsing.Optional("'") +
-                   tgrep_node_expr2 +
-                   pyparsing.ZeroOrMore("|" + tgrep_node_expr)))
+    tgrep_node_expr = (
+        tgrep_node_label_use_pred
+        | macro_use
+        | tgrep_nltk_tree_pos
+        | tgrep_qstring_icase
+        | tgrep_node_regex_icase
+        | tgrep_qstring
+        | tgrep_node_regex
+        | '*'
+        | tgrep_node_literal
+    )
+    tgrep_node_expr2 = (
+        tgrep_node_expr
+        + pyparsing.Literal('=').setWhitespaceChars('')
+        + tgrep_node_label.copy().setWhitespaceChars('')
+    ) | tgrep_node_expr
+    tgrep_node = tgrep_parens | (
+        pyparsing.Optional("'")
+        + tgrep_node_expr2
+        + pyparsing.ZeroOrMore("|" + tgrep_node_expr)
+    )
     tgrep_brackets = pyparsing.Optional('!') + '[' + tgrep_relations + ']'
     tgrep_relation = tgrep_brackets | (tgrep_op + tgrep_node)
     tgrep_rel_conjunction = pyparsing.Forward()
-    tgrep_rel_conjunction << (tgrep_relation +
-                              pyparsing.ZeroOrMore(pyparsing.Optional('&') +
-                                                   tgrep_rel_conjunction))
+    tgrep_rel_conjunction << (
+        tgrep_relation
+        + pyparsing.ZeroOrMore(pyparsing.Optional('&') + tgrep_rel_conjunction)
+    )
     tgrep_relations << tgrep_rel_conjunction + pyparsing.ZeroOrMore(
-        "|" + tgrep_relations)
+        "|" + tgrep_relations
+    )
     tgrep_expr << tgrep_node + pyparsing.Optional(tgrep_relations)
     tgrep_expr_labeled = tgrep_node_label_use + pyparsing.Optional(tgrep_relations)
     tgrep_expr2 = tgrep_expr + pyparsing.ZeroOrMore(':' + tgrep_expr_labeled)
-    macro_defn = (pyparsing.Literal('@') +
-                  pyparsing.White().suppress() +
-                  macro_name +
-                  tgrep_expr2)
-    tgrep_exprs = (pyparsing.Optional(macro_defn + pyparsing.ZeroOrMore(';' + macro_defn) + ';') +
-                   tgrep_expr2 +
-                   pyparsing.ZeroOrMore(';' + (macro_defn | tgrep_expr2)) +
-                   pyparsing.ZeroOrMore(';').suppress())
+    macro_defn = (
+        pyparsing.Literal('@') + pyparsing.White().suppress() + macro_name + tgrep_expr2
+    )
+    tgrep_exprs = (
+        pyparsing.Optional(macro_defn + pyparsing.ZeroOrMore(';' + macro_defn) + ';')
+        + tgrep_expr2
+        + pyparsing.ZeroOrMore(';' + (macro_defn | tgrep_expr2))
+        + pyparsing.ZeroOrMore(';').suppress()
+    )
     if set_parse_actions:
         tgrep_node_label_use.setParseAction(_tgrep_node_label_use_action)
         tgrep_node_label_use_pred.setParseAction(_tgrep_node_label_pred_use_action)
@@ -845,11 +958,13 @@ def _build_tgrep_parser(set_parse_actions = True):
         # relation predicates
         tgrep_expr.setParseAction(_tgrep_conjunction_action)
         tgrep_expr_labeled.setParseAction(_tgrep_segmented_pattern_action)
-        tgrep_expr2.setParseAction(functools.partial(_tgrep_conjunction_action,
-                                                     join_char = ':'))
+        tgrep_expr2.setParseAction(
+            functools.partial(_tgrep_conjunction_action, join_char=':')
+        )
         tgrep_exprs.setParseAction(_tgrep_exprs_action)
     return tgrep_exprs.ignore('#' + pyparsing.restOfLine)
 
+
 def tgrep_tokenize(tgrep_string):
     '''
     Tokenizes a TGrep search string into separate tokens.
@@ -859,6 +974,7 @@ def tgrep_tokenize(tgrep_string):
         tgrep_string = tgrep_string.decode()
     return list(parser.parseString(tgrep_string))
 
+
 def tgrep_compile(tgrep_string):
     '''
     Parses (and tokenizes, if necessary) a TGrep search string into a
@@ -869,6 +985,7 @@ def tgrep_compile(tgrep_string):
         tgrep_string = tgrep_string.decode()
     return list(parser.parseString(tgrep_string, parseAll=True))[0]
 
+
 def treepositions_no_leaves(tree):
     '''
     Returns all the tree positions in the given tree which are not
@@ -883,6 +1000,7 @@ def treepositions_no_leaves(tree):
             prefixes.add(pos[:length])
     return [pos for pos in treepositions if pos in prefixes]
 
+
 def tgrep_positions(pattern, trees, search_leaves=True):
     """
     Return the tree positions in the trees which match the given pattern.
@@ -905,11 +1023,11 @@ def tgrep_positions(pattern, trees, search_leaves=True):
                 positions = tree.treepositions()
             else:
                 positions = treepositions_no_leaves(tree)
-            yield [position for position in positions
-                      if pattern(tree[position])]
+            yield [position for position in positions if pattern(tree[position])]
         except AttributeError:
             yield []
 
+
 def tgrep_nodes(pattern, trees, search_leaves=True):
     """
     Return the tree nodes in the trees which match the given pattern.
@@ -932,7 +1050,6 @@ def tgrep_nodes(pattern, trees, search_leaves=True):
                 positions = tree.treepositions()
             else:
                 positions = treepositions_no_leaves(tree)
-            yield [tree[position] for position in positions
-                      if pattern(tree[position])]
+            yield [tree[position] for position in positions if pattern(tree[position])]
         except AttributeError:
             yield []
